#ifndef ClientDataManager_H
#define ClientDataManager_H

#include <map>
#include <string>

#include "ClientNetworkNode.h"
#include "packetID.h"
#include "Player.h"
#include "Enemy.h"
#include "Missile.h"

using namespace std;

class ClientDataManager
{
public:
	virtual ~ClientDataManager() { this->disconnect(); };

	static ClientDataManager & getSingleton();
	void init();
	void connect(string address, string playername);
	void reconnect();
	void disconnect();
	void gameOver();
	void processInput(unsigned long timeSinceInMilis);
	void update(unsigned long timeSinceInMilis);

	
	bool isInitialized() { return initialized; };

	void handlePacketConnectionAccepted(sockaddr_in * senderAddress, const char data[], const int dataSize);
	void handlePacketSendNewPlayerData(sockaddr_in * senderAddress, const char data[], const int dataSize);
	void handlePacketServerPlayerData(sockaddr_in * senderAddress, const char data[], const int dataSize);
	void handlePacketCreateEnemy(sockaddr_in * senderAddress, const char data[], const int dataSize);
	void handlePacketEnemyData(sockaddr_in * senderAddress, const char data[], const int dataSize);
	void handlePacketServerNotifiesAboutPlayerExit(sockaddr_in * senderAddress, const char data[], const int dataSize);
	void handlePacketServerCreatesMissile(sockaddr_in * senderAddress, const char data[], const int dataSize);
	void handlePacketMissileData(sockaddr_in * senderAddress, const char data[], const int dataSize);
	void handlePacketServerDestroysEnemy(sockaddr_in * senderAddress, const char data[], const int dataSize);
	void handlePacketServerChangesPlayerScore(sockaddr_in * senderAddress, const char data[], const int dataSize);
	void handlePacketID_ServerDestroysMissile(sockaddr_in * senderAddress, const char data[], const int dataSize);
	void handlePacketServerChangesPlayerLives(sockaddr_in * senderAddress, const char data[], const int dataSize);
	
	Player & getLocalPlayer() { return mLocalPlayer; };
	std::map<std::string, Player> & getOtherPlayers() { return mOtherPlayers; };

	bool isDeadReckoningActive;

	bool isPlayerFiringValid;

private:
	ClientDataManager() { initialized = false; timeSinceLastPlayerPacketUpdateInMilis = 0; };
	bool initialized;

	unsigned long timeSinceLastPlayerPacketUpdateInMilis;

	// Player data structures:
	std::string mPlayerName; // to identify player from server
	Player mLocalPlayer;

	std::map<std::string, Player> mOtherPlayers;

	// Local enemy copies (+Dead Reckoning)
	typedef std::map<string, Enemy> EnemyMapType;
	EnemyMapType mEnemies;

	// Local missile copies (+Dead Reckoning, of course)
	typedef std::map<string, Missile> MissileMapType;
	MissileMapType mMissiles;

	// networking:
	ClientNetworkNode mNetworkNode;

	int lastNumberOfBytesRead;
    char lastReadDataBuf[MAXBUFFERSIZE];

};

#endif