#ifndef ClientGameManager_H
#define ClientGameManager_H

#include <OIS/OIS.h>

#include "InputManager.h"
#include "ClientGameState.h"

class ClientGameManager : public OIS::KeyListener, OIS::MouseListener, public Ogre::WindowEventListener
{
public:
    virtual ~ClientGameManager();

    void startGame( ClientGameState *gameState );

    void changeState( ClientGameState *gameState );
    void pushState( ClientGameState *gameState );
    void popState();
    void requestShutdown();
	void processShutdown();

    static ClientGameManager & getSingleton();

protected:
	ClientGameManager();
    ClientGameManager( const ClientGameManager& ) { }
    ClientGameManager & operator = ( const ClientGameManager& );

    bool configureGame();
    void setupResources();

    bool keyPressed( const OIS::KeyEvent &e );
    bool keyReleased( const OIS::KeyEvent &e );

    bool mouseMoved( const OIS::MouseEvent &e );
    bool mousePressed( const OIS::MouseEvent &e, OIS::MouseButtonID id );
    bool mouseReleased( const OIS::MouseEvent &e, OIS::MouseButtonID id );

	// from Ogre::WindowEventListener interface class:
	bool windowClosing(Ogre::RenderWindow* rw);

	  

    Ogre::Root				* mRoot;
    Ogre::RenderWindow		* mRenderWindow;
	Ogre::SceneManager		* mgr;
	Ogre::Viewport			* vp;
	Ogre::Camera			* cam;

    bool bShutdown;
    std::vector<ClientGameState*> mStates;

    void createRoot();
	void destroyRoot();
    void terrainStuff();
    void defineResources();
    
	void loadManagers();
	void loadParticles();
    void setupRenderSystem();
       
    void createRenderWindow();
   
    void initializeResourceGroups();
    
    void setupInputSystem();
   
    void setupCEGUI();
   
    void createFrameListener();

    void startRenderLoop();

	void setupLighting();

};
#endif