
#include <winsock2.h>
#include <iostream>
#include <string>
#include <set>

#include "ClientNetworkNode.h"
#include "ClientDataManager.h"
#include "OgreConsole.h"
#include "packetID.h"

using namespace std;

// ============================================================================
bool ClientNetworkNode::hasMorePackets()
{
	if(!initialized)
		return false;

	// structure holding sender's address
	SOCKADDR_IN senderAddress;
	int senderAddrLen = sizeof(senderAddress);

	timeval timeVal;
	timeVal.tv_sec = 0;
	timeVal.tv_usec = 0;

    fd_set selectReadStruct;

    FD_ZERO(&selectReadStruct);
    FD_SET(mSocket,&selectReadStruct);
	int selret = select(mSocket+1,&selectReadStruct,0,0, &timeVal);
    if( selret < 1)
		return false;
    if(FD_ISSET(mSocket,&selectReadStruct) != 0)
    {
		//OgreConsole::getSingleton().print("ClientNetworkNode: Got a packet!");
		return true;
	}
	else
	{
		return false;
	}
}
// ============================================================================
int ClientNetworkNode::readPacket(sockaddr_in * senderAddress, char * readBuffer,
								   int readBufferMaxSize)
{
	static int clientAddrLen = sizeof(sockaddr_in);
	return recvfrom(mSocket, readBuffer,
			readBufferMaxSize, 0, (struct sockaddr *) senderAddress, &clientAddrLen);
}
// ============================================================================
void ClientNetworkNode::handlePacketConnectionAccepted(
								sockaddr_in * senderAddress,
								const char data[],
								const int dataSize)
{
	if(this->connected)
		return;

	// save our server
	mServerSockAddr = *senderAddress;

	this->connected = true;
}
// ============================================================================
void ClientNetworkNode::connect(string address, string playername)
{
	OgreConsole::getSingleton().print("Client: trying to connect to "+address+"...");
	if(!connected)
	{
		sockaddr_in server;

		server.sin_family = AF_INET;
		server.sin_addr.s_addr = inet_addr(address.c_str());
		server.sin_port = htons( 7777 );

		// lets send a Request Connection packet
		char data[1024];
		memset(data,0,1024);
		data[0] = ID_RequestConnection;
		// with desired username
		playername.copy(&data[1],playername.length());

		sendto(mSocket, data, playername.length()+1, 0, (struct sockaddr *) &server, sizeof server);
	}
	else
	{
		OgreConsole::getSingleton().print("Client: Error: you're already connected!");
	}
}
// ============================================================================
void ClientNetworkNode::reconnect(string playername)
{
	if(connected)
	{
		this->disconnect(playername);
	}

	OgreConsole::getSingleton().print("Client: trying to reconnect...");

	// lets send a Request Connection packet
	char data[1024];
	memset(data,0,1024);
	data[0] = ID_RequestConnection;
	// with desired username
	playername.copy(&data[1],playername.length());

	sendto(mSocket, data, playername.length()+1, 0, (struct sockaddr *) &mServerSockAddr, sizeof mServerSockAddr);
}
// ============================================================================
int ClientNetworkNode::sendToServer(const char * data, const int dataSize)
{
	return sendto(mSocket, data, dataSize, 0,
		(struct sockaddr *) &mServerSockAddr, sizeof mServerSockAddr );
}
// ============================================================================
void ClientNetworkNode::start()
{
	OgreConsole::getSingleton().print("ClientNetworkNode: starting...");
	if(!initialized)
		this->initSockets();
	initialized = true;
}
// ============================================================================
void ClientNetworkNode::stop()
{
	// Cleanup winsock
    WSACleanup();
}
// ============================================================================
void ClientNetworkNode::initSockets()
{
	OgreConsole::getSingleton().print("ClientNetworkNode: init sockets...");
	const int iReqWinsockVer = 2;   // Minimum winsock version required
	
	WSADATA wsaData;
	WSAStartup(MAKEWORD(iReqWinsockVer,0), &wsaData);

	/* Open a datagram socket */
	mSocket = socket(AF_INET, SOCK_DGRAM, IPPROTO_UDP);
	if (mSocket == INVALID_SOCKET)
	{
		std::cerr << "Could not create socket.\n";
		WSACleanup();
		exit(0);
	}
}
// ============================================================================
void ClientNetworkNode::disconnect( const string & playername )
{
	if(this->isConnected())
	{
		char data[ID_PlayerRequestsExit_Size];
		memset(data,0,ID_PlayerRequestsExit_Size);
		data[0] = ID_PlayerRequestsExit;
		playername.copy(&data[1],playername.length());

		this->sendToServer(data,ID_PlayerRequestsExit_Size);

		connected = false;
	}
}
// ============================================================================
ClientNetworkNode::~ClientNetworkNode()
{
	if(initialized)
	{
		this->stop();
	}
}