#include "SpringCam.h"

using namespace Ogre;

SpringCam SpringCam::mSpringCam;
// ============================================================================
// Constructor
SpringCam::SpringCam()
{
	mStiffness =2;
	mDampening = 5;
	mMass = 100;
}
// ============================================================================
///initSpringCam()
void SpringCam::initSpringCam(Vector3 camPos, Vector3 offsetPos)
{
	mCameraPos = camPos;
	mDesiredPosition = offsetPos;
}
// ============================================================================
///updateSpringCam()
Vector3 SpringCam::updateSpringCam(Vector3 camPos, Vector3 offsetPos, float _time)
{
	//Set the desired position
	mDesiredPosition = offsetPos;

	//Set the cameras current position
	mCameraPos = camPos;
    
	//Get the "spring feel" by subtracting desired position from current position and multiply by stiffness
    mStretch.x = (mCameraPos.x - mDesiredPosition.x) * -mStiffness;
    mStretch.y = (mCameraPos.y - mDesiredPosition.y) * -mStiffness;
    mStretch.z = (mCameraPos.z - mDesiredPosition.z) * -mStiffness;
    
	//Get the dampened velocity by multiplying velocity by dampening
	mDampenedVelocity.x = mVelocity.x * mDampening;
	mDampenedVelocity.y = mVelocity.y * mDampening;
	mDampenedVelocity.z = mVelocity.z * mDampening;

	//Calc the force
	mForce = mStretch - mDampenedVelocity;

	mForce = mForce;
	//Get the acceleration based on F=MA(A = F/M)
	mAcceleration.x = mForce.x * (1 / mMass);
	mAcceleration.y = mForce.y * (1 / mMass);
	mAcceleration.z = mForce.z * (1 / mMass);

	mVelocity +=(mAcceleration);
	
	//Add velocity to old camera position
	mCameraPos = mCameraPos + (mVelocity*_time);

	//Return new camera position
	return mCameraPos;
}
// ============================================================================
Vector3 SpringCam::updateSpringCam(Ogre::Vector3 camPos,Ogre::Vector3 offsetPos, float _time, float stiffness, float dampening, float mass)
{
	//Set the desired position
	mDesiredPosition = offsetPos;

	//Set the cameras current position
	mCameraPos = camPos;
    
	//Get the "spring feel" by subtracting desired position from current position and multiply by stiffness
    mStretch.x = (mCameraPos.x - mDesiredPosition.x) * -stiffness;
    mStretch.y = (mCameraPos.y - mDesiredPosition.y) * -stiffness;
    mStretch.z = (mCameraPos.z - mDesiredPosition.z) * -stiffness;
    
	//Get the dampened velocity by multiplying velocity by dampening
	mDampenedVelocity.x = mVelocity.x * dampening;
	mDampenedVelocity.y = mVelocity.y * dampening;
	mDampenedVelocity.z = mVelocity.z * dampening;

	//Calc the force
	mForce = mStretch - mDampenedVelocity;

	mForce = mForce;
	//Get the acceleration based on F=MA(A = F/M)
	mAcceleration.x = mForce.x * (1 / mass);
	mAcceleration.y = mForce.y * (1 / mass);
	mAcceleration.z = mForce.z * (1 / mass);

	mVelocity +=(mAcceleration);
	
	//Add velocity to old camera position
	mCameraPos = mCameraPos + (mVelocity*_time);

	//Return new camera position
	return mCameraPos;
}
// ============================================================================