
#include <Ogre.h>


#include "MenuState.h"
#include "PlayState.h"
#include "GuiManager.h"
#include "InputManager.h"
#include "ServerDataManager.h"

#include "OgreConsole.h"

using namespace Ogre;

MenuState * MenuState::mMenuState;

// ============================================================================
void MenuState::update( unsigned long lTimeElapsed )
{
	float timeSinceInSeconds = (float)lTimeElapsed / 1000;

	mCamera->pitch(Degree(timeSinceInSeconds));
	mCamera->yaw(Degree(timeSinceInSeconds));

	ServerDataManager::getSingleton().update(timeSinceInSeconds);
}
// ============================================================================
void MenuState::setupScene()
{
	mSceneMgr = mRoot->createSceneManager(ST_EXTERIOR_CLOSE, "MenuState SceneManager");
	mCamera = mSceneMgr->createCamera("Camera");
	mCamera->setPosition(Vector3(0,0,100));
    mCamera->lookAt(Vector3(0,0,0));
	mViewport = mRoot->getAutoCreatedWindow()->addViewport(mCamera);

	// Scene:
	mViewport->setBackgroundColour(ColourValue(0.0, 0.0, 1.0));
	mSceneMgr->setAmbientLight(ColourValue(1.0f, 1.0f, 1.0f));
	mSceneMgr->setSkyBox(true,"Examples/SpaceSkyBox");

}
// ============================================================================
void MenuState::enter( void )
{
	srand((unsigned)time(0));
	mRoot = Root::getSingletonPtr();
	this->setupScene();
	
	// Its easier to write 'gui' than 'GuiManager::getSingleton().getGuiPtr()' 
	MyGUI::Gui * gui = GuiManager::getSingleton().getGuiPtr();
	gui->setSceneManager(mSceneMgr);

	widgetPtr = MyGUI::LayoutManager::getInstance().load("ServerMenuState.layout");
	gui->findWidget<MyGUI::Widget>("Background")->setSize(gui->getViewSize());

	// OgreConsole
	OgreConsole::getSingleton().init(mRoot);
	OgreConsole::getSingleton().setVisible(false);
	//OgreConsole::getSingleton().addCommand("quit",&CMD_Quit);
	//OgreConsole::getSingleton().addCommand("screenshot",&CMD_Screenshot);

	// set MyGUI callbacks
	MyGUI::ButtonPtr start = gui->findWidget<MyGUI::Button>("StartServer");
	start->eventMouseButtonClick = MyGUI::newDelegate(this, &MenuState::buttonRequestStartServer);

	MyGUI::ButtonPtr exit = gui->findWidget<MyGUI::Button>("Exit");
	exit->eventMouseButtonClick = MyGUI::newDelegate(this, &MenuState::buttonRequestShutdown);

	InputManager::getSingleton().setMousePosition(gui->getViewSize().width/2,gui->getViewSize().height/2);

	GuiManager::getSingleton().getGuiPtr()->showPointer();
}
// ============================================================================
void MenuState::exit( void )
{
	//OgreConsole::getSingleton().shutdown();
	MyGUI::LayoutManager::getInstance().unloadLayout(widgetPtr);
	mRoot->getAutoCreatedWindow()->removeViewport(0);
}
// ============================================================================
void MenuState::pause( void )
{
}
// ============================================================================
void MenuState::resume( void )
{
}

// ============================================================================
void MenuState::keyPressed( const OIS::KeyEvent &e )
{
	// show/hide console window:
	if( e.key == OIS::KC_GRAVE || e.key == OIS::KC_KANJI) // keys: ~ or `
	{
		OgreConsole::getSingleton().setVisible( !OgreConsole::getSingleton().isVisible() );
	}

	GuiManager::getSingleton().getGuiPtr()->injectKeyPress( e );
}
// ============================================================================
void MenuState::keyReleased( const OIS::KeyEvent &e )
{
	if( e.key == OIS::KC_ESCAPE )
	{
		this->requestShutdown();
	}

	GuiManager::getSingleton().getGuiPtr()->injectKeyRelease( e );
}
// ============================================================================
void MenuState::mouseMoved( const OIS::MouseEvent &e )
{
	GuiManager::getSingleton().getGuiPtr()->injectMouseMove(e.state.X.abs,e.state.Y.abs, 300);
}
// ============================================================================
void MenuState::mousePressed( const OIS::MouseEvent &e, OIS::MouseButtonID id )
{
	GuiManager::getSingleton().getGuiPtr()->injectMousePress(e,id);
}
// ============================================================================
void MenuState::mouseReleased( const OIS::MouseEvent &e, OIS::MouseButtonID id )
{
	GuiManager::getSingleton().getGuiPtr()->injectMouseRelease(e,id);
}
// ============================================================================
MenuState* MenuState::getSingletonPtr( void )
{
	if( !mMenuState )
	{
		mMenuState = new MenuState();
	}

	return mMenuState;
}
// ============================================================================
void MenuState::buttonRequestShutdown(MyGUI::WidgetPtr _widget)
{
	this->requestShutdown();
}
// ============================================================================
void MenuState::buttonRequestStartServer(MyGUI::WidgetPtr _widget)
{
	ServerDataManager::getSingleton().init();
	this->changeState(PlayState::getSingletonPtr());
}
// ============================================================================

