
#include <Ogre.h>


#include "PlayState.h"
#include "GuiManager.h"
#include "InputManager.h"
#include "ServerDataManager.h"

#include "OgreConsole.h"

using namespace Ogre;

PlayState * PlayState::mPlayState;

// ============================================================================
void PlayState::update( unsigned long lTimeElapsedInMilis )
{
	// Process input which would change internal data
	ServerDataManager::getSingleton().processInput(lTimeElapsedInMilis);

	// Update logic and networking
	ServerDataManager::getSingleton().update(lTimeElapsedInMilis);

	// Process camera controls
	OIS::Keyboard * keyboard = InputManager::getSingleton().getKeyboard();
	OIS::Mouse * mouse = InputManager::getSingleton().getMouse();

	// Rotate the camera if right button is pressed
	if(mouse->getMouseState().buttonDown(OIS::MB_Right))
	{
		mCamera->yaw(Radian(-(float)mouse->getMouseState().X.rel / 100));
		mCamera->pitch(Radian(-(float)mouse->getMouseState().Y.rel / 100));

		if(keyboard->isKeyDown(OIS::KC_W))
		{
			mCamera->moveRelative(Vector3::UNIT_Z * -20.0f);
		}
		if(keyboard->isKeyDown(OIS::KC_S))
		{
			mCamera->moveRelative(Vector3::UNIT_Z * 20.0f);
		}
		
		if(keyboard->isKeyDown(OIS::KC_A))
		{
			mCamera->moveRelative(Vector3::UNIT_X * -20.0f);
		}
		if(keyboard->isKeyDown(OIS::KC_D))
		{
			mCamera->moveRelative(Vector3::UNIT_X * 20.0f);
		}

	}
}
// ============================================================================
void PlayState::setupScene()
{
	mSceneMgr = mRoot->createSceneManager(ST_EXTERIOR_CLOSE, "PlayState SceneManager");
	mCamera = mSceneMgr->createCamera("Camera");
	mCamera->setPosition(Vector3(0,0,100));
    mCamera->lookAt(Vector3(0,0,0));
	mViewport = mRoot->getAutoCreatedWindow()->addViewport(mCamera);

	// Scene:
	mViewport->setBackgroundColour(ColourValue(0.0, 0.0, 1.0));
	mSceneMgr->setAmbientLight(ColourValue(1.0f, 1.0f, 1.0f));
	mSceneMgr->setSkyBox(true,"Examples/SpaceSkyBox");

}
// ============================================================================
void PlayState::enter( void )
{

	srand((unsigned)time(0));
	mRoot = Root::getSingletonPtr();
	this->setupScene();
	
	// Its easier to write 'gui' than 'GuiManager::getSingleton().getGuiPtr()' 
	MyGUI::Gui * gui = GuiManager::getSingleton().getGuiPtr();
	gui->setSceneManager(mSceneMgr);

	widgetPtr = MyGUI::LayoutManager::getInstance().load("ServerPlayState.layout");

	// Server IP
	// Doesn't work
	// Get 0.0.0.0 all the time.
	//ipText = gui->createWidget<MyGUI::StaticText>("StaticText", 400, 20, 300, 80, MyGUI::ALIGN_DEFAULT, "Main");
	//unsigned int height = static_cast<MyGUI::FontPtr>(
	//	MyGUI::FontManager::getInstance().getByName("DejaVuSans") )->getDefaultHeight();
	//ipText->setFontName("DejaVuSans");
	//ipText->setFontHeight(height);
	//ipText->setTextAlign( MyGUI::Align::Left );
	//ipText->setTextColour(MyGUI::Colour(1.0f,1.0f,1.0f));
	//string serverIpString = ServerDataManager::getSingleton().getNetworkNode().getIP();
	//ipText->setCaption("Server IP: " + serverIpString);

	deadReckoningText = gui->createWidget<MyGUI::StaticText>("StaticText", 400, 20, 300, 20, MyGUI::ALIGN_DEFAULT, "Main");
	unsigned int height = static_cast<MyGUI::FontPtr>(
		MyGUI::FontManager::getInstance().getByName("DejaVuSans") )->getDefaultHeight();
	deadReckoningText->setFontName("DejaVuSans");
	deadReckoningText->setFontHeight(height);
	deadReckoningText->setTextAlign( MyGUI::Align::Left );
	deadReckoningText->setTextColour(MyGUI::Colour(1.0f,1.0f,1.0f));

	string deadActive = ServerDataManager::getSingleton().isDeadReckoningActive?"activated":"deactivated";
	deadReckoningText->setCaption("Dead Reckoning ('R'): " + deadActive);

	packetLossText = gui->createWidget<MyGUI::StaticText>("StaticText", 400, 40, 300, 80, MyGUI::ALIGN_DEFAULT, "Main");

	packetLossText->setFontName("DejaVuSans");
	packetLossText->setFontHeight(height);
	packetLossText->setTextAlign( MyGUI::Align::Left );
	packetLossText->setTextColour(MyGUI::Colour(1.0f,1.0f,1.0f));

	string lossActive = ServerDataManager::getSingleton().isPacketLossActive?"activated":"deactivated";
	packetLossText->setCaption("Packet loss ('P'): " + lossActive);

	infoText = gui->createWidget<MyGUI::StaticText>("StaticText", 400, 60, 300, 140, MyGUI::ALIGN_DEFAULT, "Main");

	infoText->setFontName("DejaVuSans");
	infoText->setFontHeight(height);
	infoText->setTextAlign( MyGUI::Align::Left );
	infoText->setTextColour(MyGUI::Colour(1.0f,0.0f,0.0f));

	infoText->setCaption("Press `/~ to show/hide client connection info");



	// OgreConsole
	OgreConsole::getSingleton().setVisible(false);

	// set MyGUI callbacks
	MyGUI::ButtonPtr createEnemy = gui->findWidget<MyGUI::Button>("CreateEnemy");
	createEnemy->eventMouseButtonClick = MyGUI::newDelegate(this, &PlayState::buttonRequestCreateEnemy);

	GuiManager::getSingleton().getGuiPtr()->showPointer();

}
// ============================================================================
void PlayState::exit( void )
{
	OgreConsole::getSingleton().shutdown();
	MyGUI::LayoutManager::getInstance().unloadLayout(widgetPtr);
	mRoot->getAutoCreatedWindow()->removeViewport(0);
}
// ============================================================================
void PlayState::pause( void )
{
}
// ============================================================================
void PlayState::resume( void )
{
}

// ============================================================================
void PlayState::keyPressed( const OIS::KeyEvent &e )
{

	// show/hide console window:
	if( e.key == OIS::KC_GRAVE || e.key == OIS::KC_KANJI) // keys: ~ or `
	{
		OgreConsole::getSingleton().setVisible( !OgreConsole::getSingleton().isVisible() );
	}

	if(e.key == OIS::KC_R)
	{
		// switch dead reckoning
		ServerDataManager::getSingleton().isDeadReckoningActive = 
			!ServerDataManager::getSingleton().isDeadReckoningActive;

		string deadActive = ServerDataManager::getSingleton().isDeadReckoningActive?"activated":"deactivated";
		deadReckoningText->setCaption("Dead Reckoning ('R'): " + deadActive);
	}

	if(e.key == OIS::KC_P)
	{
		// switch packet loss
		ServerDataManager::getSingleton().isPacketLossActive = 
			!ServerDataManager::getSingleton().isPacketLossActive;

		string lossActive = ServerDataManager::getSingleton().isPacketLossActive?"activated":"deactivated";
		packetLossText->setCaption("Packet loss ('P'): " + lossActive);
	}

	GuiManager::getSingleton().getGuiPtr()->injectKeyPress( e );
}
// ============================================================================
void PlayState::keyReleased( const OIS::KeyEvent &e )
{
	if( e.key == OIS::KC_ESCAPE )
	{
		this->requestShutdown();
	}

	GuiManager::getSingleton().getGuiPtr()->injectKeyRelease( e );
}
// ============================================================================
void PlayState::mouseMoved( const OIS::MouseEvent &e )
{
	GuiManager::getSingleton().getGuiPtr()->injectMouseMove(e.state.X.abs,e.state.Y.abs, 300);
}
// ============================================================================
void PlayState::mousePressed( const OIS::MouseEvent &e, OIS::MouseButtonID id )
{
	GuiManager::getSingleton().getGuiPtr()->injectMousePress(e,id);
}
// ============================================================================
void PlayState::mouseReleased( const OIS::MouseEvent &e, OIS::MouseButtonID id )
{
	GuiManager::getSingleton().getGuiPtr()->injectMouseRelease(e,id);
}
// ============================================================================
PlayState* PlayState::getSingletonPtr( void )
{
	if( !mPlayState )
	{
		mPlayState = new PlayState();
	}

	return mPlayState;
}
// ============================================================================
//void PlayState::buttonRequestShutdown(MyGUI::WidgetPtr _widget)
//{
//	this->requestShutdown();
//}
// ============================================================================
void PlayState::buttonRequestCreateEnemy(MyGUI::WidgetPtr _widget)
{
	ServerDataManager::getSingleton().createNewEnemy();
}
// ============================================================================

