
#include <Ogre.h>
#include <iostream>
#include <list>
#include <string>
#include <map>

#include "ServerDataManager.h"
#include "ServerNetworkNode.h"
#include "InputManager.h"
#include "packetID.h"
#include "OgreConsole.h"
#include "Player.h"
#include "Missile.h"

using namespace std;

// ============================================================================
void ServerDataManager::processInput(unsigned long timeSinceInMilis)
{
	// input processing which would change world data
	// NOT local camera etc.
}
// ============================================================================
void ServerDataManager::handlePacketTest(sockaddr_in * senderAddress, const char * buffer,
										 int bufferMaxSize)
{
	// Test/debug procedure

	// 
	// ---------------------------------------------------------------------

}
// ============================================================================
void ServerDataManager::update(unsigned long timeSinceInMilis)
{


	// Process enemies
	// but only if there's any players

	static EnemyMapType::iterator staticEnemyIter = mEnemies.begin();

	if(mPlayers.size() != 0)
	{
		EnemyMapType::iterator enemyProcessingIter = mEnemies.begin();
		while(enemyProcessingIter != mEnemies.end())
		{
			// (movement algorithms and enemy logic here)
			Enemy * enemy = &(*enemyProcessingIter).second;

			// Find the closest player
			// Here we also define enemy max sight range
			float sqrDistance;
			//float lowestSqrDistance = FLT_MAX; // infinite sight range
			float lowestSqrDistance = 250000; // this looks ok-ish
			Vector3 closestPlayerPos;
			list<Player>::iterator playerIter;
			for(playerIter = mPlayers.begin(); playerIter != mPlayers.end(); playerIter++)
			{
				sqrDistance = enemy->getPosition().squaredDistance(playerIter->getPosition());
				if(sqrDistance < lowestSqrDistance)
				{
					lowestSqrDistance = sqrDistance;
					closestPlayerPos = playerIter->getPosition();
				}
			}

			
			enemy->mSceneNode->lookAt(closestPlayerPos, Ogre::Node::TS_PARENT);

			enemy->processAccceleration(timeSinceInMilis, -1.0);
			enemy->processVelocity(timeSinceInMilis);
			enemy->processPosition(timeSinceInMilis);


			// Any hit?
			bool incrementEnemyIter = true;
			for(playerIter = mPlayers.begin(); playerIter != mPlayers.end(); ++playerIter)
			{
				float distance = enemy->getPosition().squaredDistance(playerIter->getPosition());
				float boundingRadius = playerIter->mMesh->getBoundingRadius();

				if (distance < (boundingRadius*boundingRadius) )
				{
					// We've got a hit!
					
					// Destroy the enemy
					// destroyEnemy() automatically sends the data packet
					if(staticEnemyIter == enemyProcessingIter)
					{
						++staticEnemyIter;
					}
					++enemyProcessingIter;
					incrementEnemyIter = false;
					this->destroyEnemy(enemy->getId());

					// Reduce player lives
					playerIter->setLives(playerIter->getLives() - 1);

					// Inform everyone
					char data[ID_ServerChangesPlayerLives_Size];
					playerIter->saveLivesToPacket(data);
					mNetworkNode.sendToAll(data, ID_ServerChangesPlayerLives_Size);

				}
			}

			if(incrementEnemyIter)
			{
				++enemyProcessingIter;
			}
		} // while(enemyProcessingIter != mEnemies.end())
	} //if(mPlayers.size() != 0)

	// Send the enemy data
	// Trying to limit outgoing data volume, so we're going to
	// send only a certain amount of enemy data packets each frame
	if(mEnemies.size() != 0)
	{
		for(unsigned int x = 0; 
			x < this->numberOfEnemyPacketsToSendEachFrame && x < mEnemies.size();
			++x)
		{
			if(staticEnemyIter == mEnemies.end())
			{
				staticEnemyIter = mEnemies.begin();
			}

			char data[ID_EnemyData_Size];
			(*staticEnemyIter).second.saveDataToPacket(data);
			mNetworkNode.sendToAll(data, ID_EnemyData_Size);

			++staticEnemyIter;
		}
	}

	// Process missiles
	static MissileMapType::iterator staticMissileIter = mMissiles.begin();

	MissileMapType::iterator missileProcessingIter = mMissiles.begin();
	while(missileProcessingIter != mMissiles.end())
	{
		// Movement algorithms and missile logic here
		Missile * missile = &(*missileProcessingIter).second;

		missile->processAccceleration(timeSinceInMilis, -1.0);
		missile->processVelocity(timeSinceInMilis);
		missile->processPosition(timeSinceInMilis);

		missile->mCurrentFlyingTimeInMillis += timeSinceInMilis;

		// First check if we've got enough fuel (ie flying time)
		if(missile->mCurrentFlyingTimeInMillis > 5000) // 5 seconds of flight
		{
			// Send the ServerDestroysMissile packet
			char data[ID_ServerDestroysMissile_Size];
			memset(data,0,ID_ServerDestroysMissile_Size);
			data[0] = ID_ServerDestroysMissile;
			memcpy(&data[1],missile->getId().c_str(), missile->getId().size());
			mNetworkNode.sendToAll(data,ID_ServerDestroysMissile_Size);
			
			// Destroy locally
			missile->destroy(String("PlayState SceneManager"));
			MissileMapType::iterator iterToDestroy = missileProcessingIter;
			if(staticMissileIter == missileProcessingIter)
			{
				++staticMissileIter;
			}
			++missileProcessingIter;
			mMissiles.erase(iterToDestroy);

			continue;
		}

		// Did we hit anything?
		// Missile / enemy collisions
		Vector3 missilePos = missile->getPosition();

		bool incrementMissileIterator = true;

		EnemyMapType::iterator enemyProcessingIter;
		for(enemyProcessingIter = mEnemies.begin();
			enemyProcessingIter != mEnemies.end();
			++enemyProcessingIter)
		{
			Enemy * enemy = &(*enemyProcessingIter).second;
			
			float distance = missilePos.squaredDistance(enemy->getPosition());
			float boundingRadius = enemy->mMesh->getBoundingRadius();

			if (distance < (boundingRadius*boundingRadius) )
			{
				// We've got a hit!
				
				// Destroy the enemy
				// destroyEnemy() automatically sends the data packet
				if(staticEnemyIter == enemyProcessingIter)
				{
					++staticEnemyIter;
				}
				this->destroyEnemy(enemy->getId());


				// Now, try to find the missile owner
				String ownerName = missile->getOwner();

				std::list<Player>::iterator playerIter;
				for(playerIter = mPlayers.begin(); playerIter != mPlayers.end(); playerIter++)
				{
					if((*playerIter).getPlayerName().compare(ownerName) == 0)
					{
						// Ok, we've found him. Lets increase his score
						playerIter->setScore( playerIter->getScore() + 1 );

						// Inform everyone
						char data[ID_ServerChangesPlayerScore_Size];
						playerIter->saveScoreToPacket(data);
						mNetworkNode.sendToAll(data, ID_ServerChangesPlayerScore_Size);
					}
				}

				// Send the ServerDestroysMissile packet
				char data[ID_ServerDestroysMissile_Size];
				memset(data,0,ID_ServerDestroysMissile_Size);
				data[0] = ID_ServerDestroysMissile;
				memcpy(&data[1],missile->getId().c_str(), missile->getId().size());
				mNetworkNode.sendToAll(data,ID_ServerDestroysMissile_Size);
				
				// Destroy locally
				missile->destroy(String("PlayState SceneManager"));
				MissileMapType::iterator iterToDestroy = missileProcessingIter;
				if(staticMissileIter == missileProcessingIter)
				{
					++staticMissileIter;
				}
				++missileProcessingIter;
				mMissiles.erase(iterToDestroy);
				incrementMissileIterator = false;
				break;
			} // if(enemy is hit)
		} // for (every enemy)

		// Missile / player collisions
		std::list<Player>::iterator playerIter;
		for(playerIter = mPlayers.begin();
			playerIter != mPlayers.end();
			++playerIter)
		{
			Player * player = &(*playerIter);

			// Skip missile owner - don't check with who shoots the missile!
			if(player->getPlayerName().compare(missile->getOwner()) == 0)
			{
				continue;
			}
			
			float distance = missilePos.squaredDistance(player->getPosition());
			float boundingRadius = player->mMesh->getBoundingRadius();

			if (distance < (boundingRadius*boundingRadius) )
			{
				// We've got a hit!

				// Try to find the missile owner
				String ownerName = missile->getOwner();

				std::list<Player>::iterator missileOwnerIter;
				for(missileOwnerIter = mPlayers.begin(); missileOwnerIter != mPlayers.end(); missileOwnerIter++)
				{
					if((*missileOwnerIter).getPlayerName().compare(ownerName) == 0)
					{
						// Ok, we've found him. Lets increase his score
						missileOwnerIter->setScore( missileOwnerIter->getScore() + 10 ); //higher score for PK

						// Inform everyone
						char data[ID_ServerChangesPlayerScore_Size];
						missileOwnerIter->saveScoreToPacket(data);
						mNetworkNode.sendToAll(data, ID_ServerChangesPlayerScore_Size);
					}
				}

				// Missile owner's score increased, now to decrease his target's life
				// Reduce player lives
				player->setLives(player->getLives() - 1);

				// Inform everyone about new Lives
				char changeLivesPacket[ID_ServerChangesPlayerLives_Size];
				player->saveLivesToPacket(changeLivesPacket);
				mNetworkNode.sendToAll(changeLivesPacket, ID_ServerChangesPlayerLives_Size);

				// Destroy the missile object
				// Send the ServerDestroysMissile packet
				char destroyMissilePacket[ID_ServerDestroysMissile_Size];
				memset(destroyMissilePacket,0,ID_ServerDestroysMissile_Size);
				destroyMissilePacket[0] = ID_ServerDestroysMissile;
				memcpy(&destroyMissilePacket[1],missile->getId().c_str(), missile->getId().size());
				mNetworkNode.sendToAll(destroyMissilePacket,ID_ServerDestroysMissile_Size);
				
				// Destroy locally
				missile->destroy(String("PlayState SceneManager"));
				MissileMapType::iterator iterToDestroy = missileProcessingIter;
				if(staticMissileIter == missileProcessingIter)
				{
					++staticMissileIter;
				}
				++missileProcessingIter;
				mMissiles.erase(iterToDestroy);
				incrementMissileIterator = false;
				break;
			} // if(player is hit)
		} // for (every player)

		// Ok, missile didn't hit anything and is still alive
		// Go to the next missile
		if(incrementMissileIterator)
		{
			++missileProcessingIter;
		}

	} // while(missileProcessingIter != mMissiles.end())

	// Send missile data while limiting bandwith
	if(mMissiles.size() != 0)
	{
		for(unsigned int x = 0; 
			x < this->numberOfMissilePacketsToSendEachFrame && x < mMissiles.size();
			++x)
		{
			if(staticMissileIter == mMissiles.end())
			{
				staticMissileIter = mMissiles.begin();
			}

			char data[ID_MissileData_Size];
			(*staticMissileIter).second.saveDataToPacket(data);
			mNetworkNode.sendToAll(data, ID_MissileData_Size);

			++staticMissileIter;
		}
	}
	

	// Networking, data sending
	sockaddr_in clientAddress;

	while(mNetworkNode.hasMorePackets())
	{
		// clear previous data
		memset(&clientAddress,0,sizeof(sockaddr_in));
		memset(lastReadDataBuf,0,MAXBUFFERSIZE);

		lastNumberOfBytesRead = mNetworkNode.readPacket( &clientAddress, lastReadDataBuf, MAXBUFFERSIZE);

		// ================= main packet handling ====================
		if(lastNumberOfBytesRead < 0)
		{
			std::cerr << "ServerNetworkNode error.";
		}
		else
		{
			switch(lastReadDataBuf[0])
			{
				case ID_RequestConnection:
					mNetworkNode.handlePacketRequestConnection(&clientAddress,
													lastReadDataBuf, lastNumberOfBytesRead);
					this->handlePacketRequestConnection(&clientAddress,
													lastReadDataBuf, lastNumberOfBytesRead);
					break;

				case ID_PlayerData:
					if((rand()%5) != 0 || this->isPacketLossActive == false)
					{
						this->handlePacketRawPlayerData(&clientAddress,
													lastReadDataBuf, lastNumberOfBytesRead);
					}
					break;

				case ID_PlayerRequestsExit:
					// Only if IP-PlayerName pair matches our data (to avoid hacking)
					if(mNetworkNode.handlePacketPlayerRequestsExit(&clientAddress,
													lastReadDataBuf, lastNumberOfBytesRead))
					{
						this->handlePacketPlayerRequestsExit(&clientAddress,
													lastReadDataBuf, lastNumberOfBytesRead);
					}
					break;

				case ID_PlayerRequestsCreateMissile:
					this->handlePacketPlayerRequestsCreateMissile(&clientAddress,
													lastReadDataBuf, lastNumberOfBytesRead);
					break;

				case ID_Test:
					handlePacketTest(&clientAddress,lastReadDataBuf,lastNumberOfBytesRead);

				default:
					OgreConsole::getSingleton().print("Received unhandled data: " 
						+std::string(lastReadDataBuf) );
					break;
			}
		}
		// ===========================================================

	} // while(mNetworkNode.hasMorePackets())

	// DEAD RECKONING (on server side - smooth player movement)
	// Implementing this requires both data structure planning and simple coding
	// With each packet, we update the current player velocity, rotational speeds and 
	// finally position. Without the velocity vector and rotational speeds, we
	// wouldn't be able to interpolate player's current position. But since we have them,
	// we can use simple physics to update its position despite not having a data
	// packet.
	if(this->isDeadReckoningActive)
	{
		list<Player>::iterator playerIter;
		for(playerIter = mPlayers.begin(); playerIter != mPlayers.end(); playerIter++)
		{
			Player * player = &(*playerIter);
			player->processVelocity(timeSinceInMilis);
			player->processPosition(timeSinceInMilis);
			player->processPitch(timeSinceInMilis);
			player->processRoll(timeSinceInMilis);
			player->processYaw(timeSinceInMilis);

			// Fancy visuals
			player->particleUpdate();
		}
	}

}
// ============================================================================
void ServerDataManager::handlePacketPlayerRequestsCreateMissile(sockaddr_in * clientAddress,
												 const char * data,
												 int dataSize)
{

	string ownerName(&data[1]/*skip packetID */);

	// Find the sender Player class
	std::list<Player>::iterator playerIter;
	for(playerIter = mPlayers.begin(); playerIter != mPlayers.end(); playerIter++)
	{
		if((*playerIter).getPlayerName().compare(ownerName) == 0)
		{
			// Ok, we've found him.
			Missile missile;

			string entityId = string("Missile")+Ogre::StringConverter::toString(nextMissileNumber++);

			if(!missile.mMesh)
			{
				missile.mMesh = Ogre::Root::getSingleton().getSceneManager(
					"PlayState SceneManager")->createEntity(entityId, "Missile.mesh");

				missile.mSceneNode = Ogre::Root::getSingleton().getSceneManager(
					"PlayState SceneManager")->getRootSceneNode()->createChildSceneNode(entityId+"Node");

				missile.mSceneNode->attachObject(missile.mMesh);
				missile.mSceneNode->setScale( Vector3(0.15,0.15,0.15) );
			}

			missile.setOwner(ownerName);
			
			missile.setId(entityId);

			missile.setPosition(playerIter->getPosition());
			

			// Now here we could create really awesome effects, like missiles
			// dropping under Players ship, rotating missiles,
			// strange-physics missiles etc
			// Try changing the initial Acceleration value, and also
			// the current Acceleration value, by default,
			// the missile simply accelerates forward
			// see: Missile::processAcceleration method etc.
			missile.setOrientation( playerIter->getOrientation() );
			missile.setVelocity( playerIter->getVelocity() );
			missile.setAcceleration( Vector3(0.0,0.0,0.0) );

			missile.mSceneNode->setVisible(true);

			// Now send the enemy data across the net to clients
			char data[ID_ServerCreatesMissile_Size];
			memset(data,0,ID_ServerCreatesMissile_Size);
			missile.writeInitializationDataToPacket( data );

			mNetworkNode.sendToAll(data,ID_ServerCreatesMissile_Size);

			// Store our local copy of the missile
			mMissiles.insert(make_pair(missile.getId(),missile));

			break;
		}
	}
}
// ============================================================================
void ServerDataManager::handlePacketPlayerRequestsExit(sockaddr_in * clientAddress,
												 const char * data,
												 int dataSize)
{
	std::string stringPlayerName(&data[1]/*skip packetID */);

	// Find the sender Player class
	std::list<Player>::iterator playerIter;
	for(playerIter = mPlayers.begin(); playerIter != mPlayers.end(); playerIter++)
	{
		if((*playerIter).getPlayerName().compare(stringPlayerName) == 0)
		{
			// Ok, we've found him.
			
			// Prepare a disconnection packet
			char data[ID_ServerNotifiesAboutPlayerExit_Size];
			memset(data,0,ID_ServerNotifiesAboutPlayerExit_Size);
			data[0] = ID_ServerNotifiesAboutPlayerExit;
			memcpy(&data[1],stringPlayerName.c_str(), stringPlayerName.size());

			mNetworkNode.sendToAll(data,ID_ServerNotifiesAboutPlayerExit_Size);

			// Now remove him locally
			// First, free the colour
			std::list<std::pair<std::string, bool> >::iterator colIter;
			for(colIter = mColoursTaken.begin(); colIter != mColoursTaken.end(); colIter++)
			{
				if( colIter->first.at(0) == playerIter->getColour() )
				{
					// Color taken = false
					colIter->second = false;
					break;
				}
			}

			// mMesh, mSceneNode and mParticleNode/Sys
			playerIter->mSceneNode->detachAllObjects();

			Root::getSingleton().getSceneManager(
			"PlayState SceneManager")->destroyEntity(playerIter->mMesh->getName());

			Root::getSingleton().getSceneManager(
				"PlayState SceneManager")->destroyParticleSystem(playerIter->mParticleSys);

			playerIter->mSceneNode->removeChild( playerIter->mParticleNode );

			Ogre::SceneNode* todestroy = (Ogre::SceneNode*)playerIter->mSceneNode->getParent()->removeChild( playerIter->mSceneNode->getName() );
			
			Root::getSingleton().getSceneManager(
				"PlayState SceneManager")->destroySceneNode(todestroy);



			// display debug:
			OgreConsole::getSingleton().print("Server: removed client '"
				+playerIter->getPlayerName()
				+"' from IP: "
				+std::string( inet_ntoa(clientAddress->sin_addr) )
				);

			// Now erase the whole player instance
			mPlayers.erase(playerIter);

			// Finish!
			break;
		}
	}
}
// ============================================================================
void ServerDataManager::handlePacketRawPlayerData(sockaddr_in * clientAddress,
												 const char * data,
												 int dataSize)
{
	//char playerName[MAXPLAYERNAMESIZE];
	//memset(playerName,0,MAXPLAYERNAMESIZE);
	//strcpy(playerName,&data[1] /*skip packetID */);
	std::string stringPlayerName(&data[1]/*skip packetID */);

	// Find the sender Player class
	std::list<Player>::iterator iter;
	for(iter = mPlayers.begin(); iter != mPlayers.end(); iter++)
	{
		if((*iter).getPlayerName().compare(stringPlayerName) == 0)
		{
			(*iter).readDataFromPacket(data);
			// Now send the fresh data to everyone else
			// but as another packet
			char newData[ID_ServerPlayerData_Size];
			(*iter).saveDataToPacket(newData);
			newData[0] = ID_ServerPlayerData;
			mNetworkNode.sendToAllExceptOneClient((*iter).getPlayerName(),newData,ID_ServerPlayerData_Size);
			// break the loop and return
			break;
		}
	}
}
// ============================================================================
void ServerDataManager::handlePacketRequestConnection(sockaddr_in * clientAddress,
												 const char * data,
												 int dataSize)
{
	// get the data without packetID
	std::string newPlayerName(&data[1]);

	// Find whether this is a unique name
	bool uniqueName = true;
	std::list<Player>::iterator iter;
	for(iter = mPlayers.begin(); iter != mPlayers.end(); iter++)
	{
		if((*iter).getPlayerName().compare(newPlayerName) == 0)
		{
			uniqueName = false;
			break;
		}
	}

	if(uniqueName == false)
	{
		// we need to remove it from connected clients list
		mNetworkNode.removeClient(newPlayerName);
		return;
	}

	// prepare a response
	std::string sResponse(" ");
	// set packets corresponding ID
	sResponse[0] = ID_ConnectionAccepted;
	// send colour
	char playerColour;
	std::string entityId;

	// choose a free colour
	for(list<pair<string,bool> >::iterator iter = mColoursTaken.begin();
		iter != mColoursTaken.end();
		iter++)
	{
		// Free colour?
		if((*iter).second != true)
		{
			playerColour = (*iter).first.at(0);
			(*iter).second = true;
			break;
		}
	}
	switch(playerColour)
	{
		case 'G':
			entityId = "OortGreen";
			break;
		case 'R':
			entityId = "OortRed";
			break;
		case 'W':
			entityId = "OortWhite";
			break;
	}
	std::string sColour(" ");
	sColour[0] = playerColour;
	sResponse.append(sColour);

	// send back registered nickname
	sResponse.append(newPlayerName);

	// inform the client
	mNetworkNode.sendToClient( clientAddress, sResponse.c_str(), sResponse.length() );

	// Now we physically create the Player instance AND send ANOTHER packet!
	this->addPlayer(playerColour,newPlayerName,entityId);

	// display debug:
	OgreConsole::getSingleton().print("Server: added client '"
		+newPlayerName
		+"' from IP: "
		+std::string( inet_ntoa(clientAddress->sin_addr) )
		);
}
// ============================================================================
void ServerDataManager::init()
{
	if(!initialized)
	{
		mNetworkNode.start(7777);

		mColoursTaken.push_back(std::make_pair<string,bool>("G",false));
		mColoursTaken.push_back(std::make_pair<string,bool>("R",false));
		mColoursTaken.push_back(std::make_pair<string,bool>("W",false));

		initialized = true;
	}
}
// ============================================================================
void ServerDataManager::addPlayer(char colour, string playerName, string entityId)
{
	Player player;

	if(!player.mMesh)
	{
		player.mMesh = Ogre::Root::getSingleton().getSceneManager(
			"PlayState SceneManager")->createEntity(entityId, "Oort.mesh");

		player.mSceneNode = Ogre::Root::getSingleton().getSceneManager(
			"PlayState SceneManager")->getRootSceneNode()->createChildSceneNode(entityId+"Node");

		player.mSceneNode->attachObject(player.mMesh);
		player.mSceneNode->setScale( Vector3(1.0,1.0,1.0) );
	}

	if(player.mSceneNode)
	{
		player.mParticleNode = player.mSceneNode->createChildSceneNode();
		player.mParticleSys = Ogre::Root::getSingleton().getSceneManager(
			"PlayState SceneManager")->createParticleSystem(entityId+"Particle", "tail2");
		player.mParticleNode->setPosition(Vector3(0, -10, 70));
		player.mParticleNode->attachObject(player.mParticleSys);
	}
	
	player.setId(entityId);
	player.setPlayerName(playerName);
	player.setColour(colour);
	// Random position 
	player.setPosition(
		Vector3(Ogre::Math::RangeRandom(-100,100),Ogre::Math::RangeRandom(-100,100),-1000));
	
	player.setOrientation( Quaternion() );
	player.setVelocity( Vector3(0.0,0.0,0.0) );
	player.setAcceleration( Vector3(0.0,0.0,0.0) );
	player.setCurrentPitchRotationSpeed(0);
	player.setCurrentRollRotationSpeed(0);
	player.setCurrentYawRotationSpeed(0);

	player.setScore(0);
	player.setLives(10);

	player.mSceneNode->setVisible(true);

	Ogre::Root::getSingleton().getSceneManager(
		"PlayState SceneManager")->getCamera("Camera")->lookAt(player.getPosition());

	// Now send the player data across the net to other clients
	char data[ID_SendNewPlayerData_Size];
	memset(data,0,ID_SendNewPlayerData_Size);
	player.writeInitializationDataToPacket( data );

	mNetworkNode.sendToAll(data,ID_SendNewPlayerData_Size);

	// We also want this player to know all other players
	for(std::list<Player>::iterator iter = mPlayers.begin();
		iter != mPlayers.end();
		iter++)
	{
		memset(data,0,ID_SendNewPlayerData_Size);
		(*iter).writeInitializationDataToPacket( data );
		mNetworkNode.sendToClient(player.getPlayerName(), data, ID_SendNewPlayerData_Size);
	}

	char enemyData[ID_ServerCreatesEnemy_Size];
	// ...and we want him to know all existing enemies
	for(EnemyMapType::iterator iter = mEnemies.begin();
		iter != mEnemies.end();
		iter++)
	{
		memset(enemyData,0,ID_ServerCreatesEnemy_Size);
		(*iter).second.writeInitializationDataToPacket(enemyData);
		mNetworkNode.sendToClient(player.getPlayerName(), enemyData, ID_ServerCreatesEnemy_Size);
	}

	// Store our local copy of the player
	mPlayers.push_back(player);
}
// ============================================================================
string ServerDataManager::createNewEnemy()
{
	Enemy enemy;

	string entityId = string("FrogTank")+Ogre::StringConverter::toString(nextEnemyNumber++);

	if(!enemy.mMesh)
	{
		enemy.mMesh = Ogre::Root::getSingleton().getSceneManager(
			"PlayState SceneManager")->createEntity(
			string("FrogTank")+
			Ogre::StringConverter::toString(nextEnemyNumber), "FrogTank.mesh");

		enemy.mSceneNode = Ogre::Root::getSingleton().getSceneManager(
			"PlayState SceneManager")->getRootSceneNode()->createChildSceneNode(entityId+"Node");

		enemy.mSceneNode->attachObject(enemy.mMesh);
		enemy.mSceneNode->setScale( Vector3(1.0,1.0,1.0) );
	}
	
	enemy.setId(entityId);

	// Random position 
	enemy.setPosition(
		Vector3(Ogre::Math::RangeRandom(-2500,2500),
				Ogre::Math::RangeRandom(-2500,2500),
				Ogre::Math::RangeRandom(-2500,2500)));
	
	enemy.setOrientation( Quaternion() );
	enemy.setVelocity( Vector3(0.0,0.0,0.0) );
	enemy.setAcceleration( Vector3(0.0,0.0,0.0) );

	enemy.mSceneNode->setVisible(true);

	// Now send the enemy data across the net to other clients
	char data[ID_ServerCreatesEnemy_Size];
	memset(data,0,ID_ServerCreatesEnemy_Size);
	enemy.writeInitializationDataToPacket( data );

	mNetworkNode.sendToAll(data,ID_ServerCreatesEnemy_Size);

	// Store our local copy of the enemy
	mEnemies.insert(make_pair(enemy.getId(),enemy));

	return enemy.getId();
}
// ============================================================================
ServerDataManager & ServerDataManager::getSingleton()
{
	static ServerDataManager instance;
	return instance;
}
// ============================================================================
ServerDataManager::ServerDataManager()
{
	initialized = false;
	nextEnemyNumber = 1;
	numberOfEnemyPacketsToSendEachFrame = 10;
	nextMissileNumber = 1;
	numberOfMissilePacketsToSendEachFrame = 10;
	

	isPacketLossActive = false;
	isDeadReckoningActive = true;
}
// ============================================================================
void ServerDataManager::destroyEnemy(string entityId)
{
	EnemyMapType::iterator enemyProcessingIter = mEnemies.find(entityId);
	if(enemyProcessingIter != mEnemies.end())
	{
		Enemy * enemy = &(*enemyProcessingIter).second;

		// Send the ServerDestroysMissile packet
		char data[ID_ServerDestroysEnemy_Size];
		memset(data,0,ID_ServerDestroysEnemy_Size);
		data[0] = ID_ServerDestroysEnemy;
		memcpy(&data[1],enemy->getId().c_str(), enemy->getId().size());
		mNetworkNode.sendToAll(data,ID_ServerDestroysEnemy_Size);

		// Destroy locally
		enemy->destroy(String("PlayState SceneManager"));
		mEnemies.erase(enemyProcessingIter);
	}
}
// ============================================================================