
#include <OgreException.h>

#include "ServerGameManager.h"
#include "ServerGameState.h"
#include "GuiManager.h"
#include "OgreConsole.h"

using namespace Ogre;

// ============================================================================
ServerGameManager::ServerGameManager() :
    mRoot( 0 ),
   bShutdown( false )	
{
	// nuthin'
}
// ============================================================================
ServerGameManager::~ServerGameManager()
{
	//destroyRoot();
}
// ============================================================================
void ServerGameManager::destroyRoot()
{
	if( mRoot )
	{
		mRoot->shutdown();
        delete mRoot;
        mRoot = 0;
    }
}
// ============================================================================
void ServerGameManager::startGame( ServerGameState *gameState)
{
	srand (time(0));
  	createRoot();
	defineResources();
	setupRenderSystem();
	createRenderWindow();

	initializeResourceGroups();
	setupInputSystem();

	new OgreConsole;

	this->changeState( gameState );

	 static unsigned long lTimeLastFrame = 0;

	 while( !bShutdown )
	 {
        // Calculate time since last frame and remember current time for next frame
        unsigned long lTimeCurrentFrame = mRoot->getTimer()->getMilliseconds();
        unsigned long lTimeSinceLastFrame = lTimeCurrentFrame - lTimeLastFrame;
        lTimeLastFrame = lTimeCurrentFrame;

        // Update inputmanager
        InputManager::getSingleton().capture();

        // Update current state
        mStates.back()->update( lTimeSinceLastFrame );

		WindowEventUtilities::messagePump();//yys
		// Render next frame
        mRoot->renderOneFrame();
	 }

	 this->processShutdown();
}
// ============================================================================
void ServerGameManager::changeState( ServerGameState *gameState )
{
    // Cleanup the current state
    if( !mStates.empty() ) {
        mStates.back()->exit();
        mStates.pop_back();
    }

    // Store and init the new state
    mStates.push_back( gameState );
    mStates.back()->enter();
}
// ============================================================================
void ServerGameManager::pushState( ServerGameState *gameState )
{
    // Pause current state
    if( !mStates.empty() ) {
        mStates.back()->pause();
    }

    // Store and init the new state
    mStates.push_back( gameState );
    mStates.back()->enter();
}
// ============================================================================
void ServerGameManager::popState()
{
    // Cleanup the current state
    if( !mStates.empty() ) {
        mStates.back()->exit();
        mStates.pop_back();
    }

    // Resume previous state
    if( !mStates.empty() ) {
        mStates.back()->resume();
    }
}
// ============================================================================
void ServerGameManager::requestShutdown()
{
    bShutdown = true;
}
// ============================================================================
bool ServerGameManager::keyPressed( const OIS::KeyEvent &e )
{
    // Call keyPressed of current state
    mStates.back()->keyPressed( e );

    return true;
}
// ============================================================================
bool ServerGameManager::keyReleased( const OIS::KeyEvent &e )
{
    // Call keyReleased of current state
    mStates.back()->keyReleased( e );

    return true;
}
// ============================================================================
bool ServerGameManager::mouseMoved( const OIS::MouseEvent &e )
{
    // Call mouseMoved of current state
    mStates.back()->mouseMoved( e );

    return true;
}
// ============================================================================
bool ServerGameManager::mousePressed( const OIS::MouseEvent &e, OIS::MouseButtonID id )
{
    // Call mousePressed of current state
    mStates.back()->mousePressed( e, id );

    return true;
}
// ============================================================================
bool ServerGameManager::mouseReleased( const OIS::MouseEvent &e, OIS::MouseButtonID id )
{
    // Call mouseReleased of current state
    mStates.back()->mouseReleased( e, id );

    return true;
}
// ============================================================================
ServerGameManager & ServerGameManager::getSingleton()
{
	static ServerGameManager instance;
    return instance;
}
// ============================================================================
void ServerGameManager::createRoot()
{
	//mRoot = new Root();
	mRoot = new Root("plugins.cfg", "ogre.cfg", ""); // test to disable writing to cfg files so can run on CD.
}
// ============================================================================
void ServerGameManager::defineResources()
{
	String secName, typeName, archName;
	ConfigFile cf;
	cf.load("resources.cfg");

	ConfigFile::SectionIterator seci = cf.getSectionIterator();
	
	while (seci.hasMoreElements())
	{
		secName = seci.peekNextKey();
		ConfigFile::SettingsMultiMap *settings = seci.getNext();
		ConfigFile::SettingsMultiMap::iterator i;
		for (i = settings->begin(); i != settings->end(); ++i)
		{
			typeName = i->first;
			archName = i->second;
			ResourceGroupManager::getSingleton().addResourceLocation(archName, typeName, secName);
		}
	}
}
// ============================================================================
void ServerGameManager::setupRenderSystem()
{
	if (!mRoot->showConfigDialog())
		throw Exception(52, "User canceled the config dialog!", "ServerGameManager::setupRenderSystem()");

}
// ============================================================================
void ServerGameManager::createRenderWindow()
{
	 mRenderWindow = mRoot->initialise( true, "Server - Ogre3DNetworkGame" );
}
// ============================================================================
void ServerGameManager::initializeResourceGroups()
{
	//TextureManager::getSingleton().setDefaultNumMipmaps(5);
	ResourceGroupManager::getSingleton().initialiseAllResourceGroups();
}
// ============================================================================
void ServerGameManager::setupInputSystem()
{
	mRenderWindow = mRoot->getAutoCreatedWindow();
    InputManager::getSingleton().initialise( mRenderWindow );
	WindowEventUtilities::addWindowEventListener( mRenderWindow, this );//yys

    InputManager::getSingleton().addKeyListener( this, "ServerGameManager" );
    InputManager::getSingleton().addMouseListener( this, "ServerGameManager" );

}
// ============================================================================
void ServerGameManager::setupLighting()
{
	SceneManager *mgr = mRoot->getSceneManager("PlayState SceneManager");
	mgr->setAmbientLight(ColourValue(1.0f, 1.0f, 1.0f));
}
// ============================================================================
void ServerGameManager::startRenderLoop()
{
	mRoot->startRendering();
}
// ============================================================================
bool ServerGameManager::windowClosing(RenderWindow* rw)
{
	requestShutdown();
	return true;
}
// ============================================================================
void ServerGameManager::processShutdown()
{
	// Clean up all the states
    while( !mStates.empty() )
	{
        mStates.back()->exit();
        mStates.pop_back();
    }
}
// ============================================================================
