
#include <winsock2.h>
#include <iostream>
#include <string>
#include <algorithm>

#include "ServerNetworkNode.h"
#include "ServerDataManager.h"
#include "ConnectionSet.h"
#include "OgreConsole.h"
#include "packetID.h"

// ============================================================================
bool ServerNetworkNode::hasMorePackets()
{
	if(!initialized)
		return false;

	// structure holding client's address
	SOCKADDR_IN clientAddress;
	int clientAddrLen = sizeof(clientAddress);

	timeval timeVal;
	timeVal.tv_sec = 0;
	timeVal.tv_usec = 0;

    fd_set selectReadStruct;

    FD_ZERO(&selectReadStruct);
    FD_SET(mSocket,&selectReadStruct);
	int selret = select(mSocket+1,&selectReadStruct,0,0, &timeVal);
    if( selret < 1)
		return false;

	if(FD_ISSET(mSocket,&selectReadStruct) != 0)
    {
		//OgreConsole::getSingleton().print("ServerNetworkNode: Got a packet!");
		return true;
	}
	else
	{
		return false;
	}

}
// ============================================================================
int ServerNetworkNode::readPacket(sockaddr_in * senderAddress, char * readBuffer,
								   int readBufferMaxSize)
{
	static int clientAddrLen = sizeof(sockaddr_in);
	return recvfrom(mSocket, readBuffer,
			readBufferMaxSize, 0, (struct sockaddr *) senderAddress, &clientAddrLen);
}
// ============================================================================
bool ServerNetworkNode::handlePacketRequestConnection(
								sockaddr_in * clientAddress,
								const char * data,
								int dataSize)
{
	// Can we add another player?
	if(mConnectedClientsMap.size() >= maxClients)
		return false;

	// get the data without packetID
	std::string newPlayerName(data+1,min<int>(dataSize-1,MAXPLAYERNAMESIZE));

	// store a new client
	mConnectedClientsMap.insert(make_pair<string,sockaddr_in>(newPlayerName,*clientAddress));

	return true;
}
// ============================================================================
bool ServerNetworkNode::handlePacketPlayerRequestsExit(
								sockaddr_in * clientAddress,
								const char * data,
								int dataSize)
{
	bool isValid = false;

	// get the data without packetID
	//char charPlayerName[MAXPLAYERNAMESIZE];
	//memset(charPlayerName,0,MAXPLAYERNAMESIZE);
	//memcpy(charPlayerName, &data[1], MAXPLAYERNAMESIZE);
	std::string playerName(&data[1]);//,min<int>(dataSize-1,MAXPLAYERNAMESIZE));

	// store a new client
	std::map<std::string, sockaddr_in>::iterator iter;
	iter = mConnectedClientsMap.find(playerName);
	if(iter != mConnectedClientsMap.end())
	{
		if(iter->second.sin_addr.S_un.S_addr == 
			clientAddress->sin_addr.S_un.S_addr)
		{
			// Valid disconnect request
			mConnectedClientsMap.erase(iter);
			isValid = true;
		}
	}

	return isValid;
}
// ============================================================================
int ServerNetworkNode::sendToClient(const sockaddr_in * clientAddress, const char * data,
									const int dataSize)
{
	return sendto(mSocket, data, dataSize ,0,
		(struct sockaddr *) clientAddress, sizeof (*clientAddress) );
}
// ============================================================================
int ServerNetworkNode::sendToClient(const std::string clientName, const char * data,
									const int dataSize)
{
	map<string,sockaddr_in>::iterator iter = mConnectedClientsMap.find(clientName);
	if(iter != mConnectedClientsMap.end())
	{
		return sendToClient(&(*iter).second, data, dataSize);
	}
	else
	{
		return -1;
	}
}
// ============================================================================
void ServerNetworkNode::sendToAll(const char * data, const int dataSize)
{
	map<string,sockaddr_in>::iterator iter;
	for(iter = mConnectedClientsMap.begin(); iter != mConnectedClientsMap.end(); ++iter)
	{
		sendToClient( &(*iter).second, data, dataSize );
	}
}
// ============================================================================
void ServerNetworkNode::sendToAllExceptOneClient(const sockaddr_in * clientAddressNotToSend,
												 const char * data,
												const int dataSize)
{
	map<string,sockaddr_in>::iterator iter;
	for(iter = mConnectedClientsMap.begin(); iter != mConnectedClientsMap.end(); ++iter)
	{
		if((*iter).second.sin_addr.S_un.S_addr == 
						clientAddressNotToSend->sin_addr.S_un.S_addr)
			continue;
		sendToClient( &(*iter).second, data, dataSize );
	}
}
// ============================================================================
void ServerNetworkNode::sendToAllExceptOneClient(const std::string clientNameNotToSend,
												 const char * data, 
												const int dataSize)
{
	// First get his IP
	map<string,sockaddr_in>::iterator iter = mConnectedClientsMap.find(clientNameNotToSend);
	
	if(iter != mConnectedClientsMap.end())
	{
		// we found him
		sendToAllExceptOneClient(&(*iter).second, data, dataSize);
	}
	else
	{
		// Name not found, send to everybody 
		sendToAll(data,dataSize);
	}
}
// ============================================================================
void ServerNetworkNode::start(u_short port)
{
	OgreConsole::getSingleton().print("ServerNetworkNode: starting...");
	mPortNumber = port;
	if(!initialized)
		this->initSockets();
	initialized = true;
}
// ============================================================================
void ServerNetworkNode::stop()
{
	// Cleanup winsock
    WSACleanup();
}
// ============================================================================
void ServerNetworkNode::initSockets()
{
	OgreConsole::getSingleton().print("ServerNetworkNode: init sockets...");
	const int iReqWinsockVer = 2;   // Minimum winsock version required
	
	WSADATA wsaData;
	WSAStartup(MAKEWORD(iReqWinsockVer,0), &wsaData);

	/* Open a datagram socket */
	mSocket = socket(AF_INET, SOCK_DGRAM, IPPROTO_UDP);
	if (mSocket == INVALID_SOCKET)
	{
		std::cerr << "Could not create socket.\n";
		WSACleanup();
		exit(0);
	}

	//DWORD yes = 1;
	//if(setsockopt(UdpMasterConnectionSet.listenerConnectionID, SOL_SOCKET, SO_REUSEADDR, &yes, sizeof(int)) == -1)
 //   {
	//	WSACleanup();
	//	std::cerr << "Server-setsockopt() error.\n";
	//	exit(0);
 //   }

	/* Clear out server struct */
	memset((void *)&mServerSockAddr, '\0', sizeof(struct sockaddr_in));

	/* Set family and port */
	mServerSockAddr.sin_family = AF_INET;

	// We're the server, so our local address doesn't matter.
	// Accept every other address.
	mServerSockAddr.sin_addr.s_addr = INADDR_ANY;
	mServerSockAddr.sin_port = htons(mPortNumber);
	memset(&(mServerSockAddr.sin_zero), '\0', 8);

	/* Bind address to socket */
	if (bind(mSocket, (struct sockaddr *)&mServerSockAddr,
										sizeof(struct sockaddr_in)) == -1)
	{
		std::cerr << "Could not bind name to socket.\n";
		closesocket(mSocket);
		WSACleanup();
		exit(0);
	}
}
// ============================================================================
void ServerNetworkNode::removeClient(std::string nameToRemove)
{
	//std::list<ConnectedClient>::iterator iter;
	//for(iter = mConnectedClients.begin(); iter != mConnectedClients.end(); iter++)
	//{
	//	if((*iter).name.compare(nameToRemove) == 0)
	//	{
	//		mConnectedClients.erase(iter);
	//		return;
	//	}
	//}

	mConnectedClientsMap.erase(nameToRemove);
}
// ============================================================================
