
#include <Ogre.h>
#include <string.h>
#include <Winsock2.h>

#include "Enemy.h"
#include "packetID.h"
#include "ParsingUtils.h"

using namespace Ogre;
// ============================================================================
Quaternion Enemy::findRotationQuaternion(Vector3 destinationPoint)
{
	//Vector3 directionVec = destinationPoint - this->getPosition();
 //   directionVec.normalise();

 //   Vector3 initialRot = this->getOrientation()* Vector3::UNIT_Z;
	//Vector3 initialVec = this->mSceneNode->getInitialOrientation() * Vector3::UNIT_Z;
	//Quaternion initialQuat = initialRot.getRotationTo(initialVec);
	//
	//Vector3 src = this->mSceneNode->getInitialOrientation() * Vector3::UNIT_Z;
	//Ogre::Quaternion quat = src.getRotationTo(directionVec);
	//Quaternion totalQuat = initialQuat * quat;
	//return totalQuat;

	Vector3 directionVec = destinationPoint - this->getPosition();
	directionVec.normalise();

	Vector3 src = this->getOrientation() * Vector3::UNIT_Z;
	src.normalise();

	Quaternion quat = src.getRotationTo(directionVec);

	return quat;
}
// ============================================================================
void Enemy::processAccceleration(unsigned long timeSinceInMilis, float multiplier)
{
	Ogre::Vector3 globalFacingDirection = mSceneNode->getOrientation() * Ogre::Vector3::UNIT_Z;
	globalFacingDirection.normalise();

	Ogre::Vector3 globalAcceleration = multiplier * globalFacingDirection * 
								mMaxAcceleration * (float)(timeSinceInMilis)/1000;

	mAcceleration +=  globalAcceleration;

	if(mAcceleration.length() > mMaxAcceleration)
	{
		mAcceleration.normalise();
		mAcceleration *= mMaxAcceleration;
	}
}
// ============================================================================
void Enemy::processVelocity(unsigned long timeSinceInMilis)
{
	mVelocity += mAcceleration;
	
	if(mVelocity.length() >= mMaxSpeed)
	{
		mVelocity.normalise();
		mVelocity *= mMaxSpeed;
	}
}
// ============================================================================
void Enemy::processPosition(unsigned long timeSinceInMilis)
{
	mSceneNode->translate(mVelocity);
}
// ============================================================================
void Enemy::readDataFromPacket(const char * networkOrderedBuffer )
{
	int cnt = 0;
	// skip PacketID
	cnt += sizeof(char);

	char entID[MAXENTITYIDSIZE];
	memset(entID,0,MAXENTITYIDSIZE);
	memcpy(entID,&networkOrderedBuffer[cnt],MAXENTITYIDSIZE);
	std::string sEntId = std::string(entID);
	cnt += MAXENTITYIDSIZE;

	// Wrong receiver!
	if(mId.compare(sEntId) != 0)
		return;

	// Position
	Ogre::Vector3 position;
	ParsingUtils::parseNetworkedVector3ToHost(position,&networkOrderedBuffer[cnt]);
	cnt += 3*sizeof(float);
	mSceneNode->setPosition(position);

	// Orientation
	Ogre::Quaternion orientation;
	ParsingUtils::parseNetworkedQuaternionToHost(orientation,&networkOrderedBuffer[cnt]);
	cnt += 4*sizeof(float);
	mSceneNode->setOrientation(orientation);

	// Velocity
	ParsingUtils::parseNetworkedVector3ToHost(mVelocity,&networkOrderedBuffer[cnt]);
	cnt += 3*sizeof(float);

	// Acceleration
	ParsingUtils::parseNetworkedVector3ToHost(mAcceleration,&networkOrderedBuffer[cnt]);
	cnt += 3*sizeof(float);
}
// ============================================================================
void Enemy::saveDataToPacket( char * networkOrderedBuffer )
{
	int cnt = 0;

	networkOrderedBuffer[cnt] = ID_EnemyData;
	cnt += sizeof(char);

	// Enemy ID
	memset(&networkOrderedBuffer[cnt],0,MAXENTITYIDSIZE);
	memcpy(&networkOrderedBuffer[cnt], mId.c_str(), mId.length());
	cnt += MAXENTITYIDSIZE;

	// Position
	ParsingUtils::parseHostVector3ToNetwork(&networkOrderedBuffer[cnt],
		Ogre::Vector3(mSceneNode->getPosition()));
	cnt += 3*sizeof(float);

	// Orientation
	ParsingUtils::parseHostQuaternionToNetwork(&networkOrderedBuffer[cnt],
		Ogre::Quaternion(mSceneNode->getOrientation()));
	cnt += 4*sizeof(float);
	
	// Velocity
	ParsingUtils::parseHostVector3ToNetwork(&networkOrderedBuffer[cnt],mVelocity);
	cnt += 3*sizeof(float);

	// Acceleration
	ParsingUtils::parseHostVector3ToNetwork(&networkOrderedBuffer[cnt],mAcceleration);
	cnt += 3*sizeof(float);
}
// ============================================================================
bool Enemy::writeInitializationDataToPacket( char * networkOrderedBuffer )
{
	int cnt = 0;

	networkOrderedBuffer[cnt] = ID_ServerCreatesEnemy;
	cnt += sizeof(char);

	// Entity ID
	memset(&networkOrderedBuffer[cnt],0,MAXENTITYIDSIZE);
	memcpy(&networkOrderedBuffer[cnt], mId.c_str(), mId.length());
	cnt += MAXENTITYIDSIZE;

	// Position
	ParsingUtils::parseHostVector3ToNetwork(&networkOrderedBuffer[cnt],
		Ogre::Vector3(mSceneNode->getPosition()));
	cnt += 3*sizeof(float);

	// Orientation
	ParsingUtils::parseHostQuaternionToNetwork(&networkOrderedBuffer[cnt],
		Ogre::Quaternion(mSceneNode->getOrientation()));
	cnt += 4*sizeof(float);
	
	// Velocity
	ParsingUtils::parseHostVector3ToNetwork(&networkOrderedBuffer[cnt],mVelocity);
	cnt += 3*sizeof(float);

	// Acceleration
	ParsingUtils::parseHostVector3ToNetwork(&networkOrderedBuffer[cnt],mAcceleration);
	cnt += 3*sizeof(float);

	// yeah, its like SaveDataToPacket but could do more, like send some initial score, lives etc

	// Yay!
	return true;
}
// ============================================================================
bool Enemy::initializeFromPacket( const char * networkOrderedBuffer, Ogre::String sceneManager )
{
	int cnt = 0;
	// skip PacketID
	cnt += sizeof(char);

	// Entity ID + mesh and SceneNode init
	char entityId[MAXENTITYIDSIZE];
	memset(entityId,0,MAXENTITYIDSIZE);
	memcpy(entityId,&networkOrderedBuffer[cnt],MAXENTITYIDSIZE);
	mId = std::string(entityId);
	cnt += MAXENTITYIDSIZE;

	if(!mMesh)
	{
		mMesh = Ogre::Root::getSingleton().getSceneManager(
			sceneManager)->createEntity(mId, "FrogTank.mesh");
	}

	if(!mSceneNode)
	{
		mSceneNode = Ogre::Root::getSingleton().getSceneManager(
			sceneManager)->getRootSceneNode()->createChildSceneNode(mId+"Node");

		mSceneNode->attachObject(mMesh);
		mSceneNode->setScale( Ogre::Vector3(1.0,1.0,1.0) );
	}

	// Position
	Ogre::Vector3 position;
	ParsingUtils::parseNetworkedVector3ToHost(position,&networkOrderedBuffer[cnt]);
	cnt += 3*sizeof(float);
	mSceneNode->setPosition(position);

	// Orientation
	Ogre::Quaternion orientation;
	ParsingUtils::parseNetworkedQuaternionToHost(orientation,&networkOrderedBuffer[cnt]);
	cnt += 4*sizeof(float);
	mSceneNode->setOrientation(orientation);

	// Velocity
	ParsingUtils::parseNetworkedVector3ToHost(mVelocity,&networkOrderedBuffer[cnt]);
	cnt += 3*sizeof(float);

	// Acceleration
	ParsingUtils::parseNetworkedVector3ToHost(mAcceleration,&networkOrderedBuffer[cnt]);
	cnt += 3*sizeof(float);

	mIsInitialized = true;
	
	// No problems at all 
	return true;
}
// ============================================================================
void Enemy::setPosition(Ogre::Vector3 & _position)
{
	mSceneNode->setPosition(_position);
}
// ============================================================================
void Enemy::setOrientation(Ogre::Quaternion & _orientation)
{
	mSceneNode->setOrientation(_orientation);
}
// ============================================================================
Enemy::Enemy(void) :
	mClassName("Enemy"),
	mMesh(0),
	mSceneNode(0),
	mMaxSpeed(8),
	mMaxAcceleration(2),
	mId(""),
	mIsInitialized(false)
{
}
// ============================================================================
void Enemy::setId(Ogre::String _id)
{
	mId = _id;
}
// ============================================================================
void Enemy::setVelocity(Ogre::Vector3 & _velocity)
{
	mVelocity = _velocity;
}
// ============================================================================
void Enemy::setAcceleration(Ogre::Vector3 & _acceleration)
{
	mAcceleration = _acceleration;
}
// ============================================================================
Enemy & Enemy::operator=(Enemy & _other)
{
	mMesh = _other.mMesh;
	mSceneNode = _other.mSceneNode;

	mId = _other.mId;

	mVelocity = _other.mVelocity;
	mAcceleration = _other.mAcceleration;

	mIsInitialized = _other.mIsInitialized;

	return *this;
}
// ============================================================================
void Enemy::destroy(Ogre::String & sceneManagerName)
{
	SceneManager * sceneMgr = Root::getSingleton().getSceneManager(sceneManagerName);
	
	// mMesh, mSceneNode
	mSceneNode->detachAllObjects();

	sceneMgr->destroyEntity(mMesh->getName());

	SceneNode* todestroy = (SceneNode*)mSceneNode->getParent()->removeChild( mSceneNode->getName());

	sceneMgr->destroySceneNode(todestroy);
}
// ============================================================================