
#ifndef Enemy_H
#define Enemy_H

#include <Ogre.h>

class Enemy
{
public:
	Enemy(void);
	~Enemy(void) {};

	void destroy(Ogre::String & sceneManagerName);

	const Ogre::String	mClassName;

	Ogre::Entity		*mMesh;
	Ogre::SceneNode		*mSceneNode;

	Ogre::String	getId() { return mId; };
	void			setId(Ogre::String _id);

	Ogre::Vector3	getPosition() { return mSceneNode->getPosition(); };
	void			setPosition(Ogre::Vector3 & _position);

	Ogre::Quaternion getOrientation() { return mSceneNode->getOrientation(); };
	void			setOrientation(Ogre::Quaternion & _orientation);

	Ogre::Vector3	getVelocity() { return mVelocity; };
	void			setVelocity(Ogre::Vector3 & _velocity);

	Ogre::Vector3	getAcceleration() { return mAcceleration; }
	void			setAcceleration(Ogre::Vector3 & _acceleration);

	// Accelerate in our local direction
	void			processAccceleration(unsigned long timeSinceInMilis, float multiplier = 1.0f);

	// Change the velocity according to the new acceleration
	void			processVelocity(unsigned long timeSinceInMilis);
	
	// Apply the velocity to change position
	void			processPosition(unsigned long timeSinceInMilis);

	// Find the rotation quaternion to the target point
	Ogre::Quaternion findRotationQuaternion(Ogre::Vector3 destinationPoint);

	// Creating an enemy from a network packet!
	bool initializeFromPacket( const char * networkOrderedBuffer, Ogre::String sceneManager );

	bool isInitialized() { return mIsInitialized; };

	// Send the creation (position, ID, etc) through the network
	bool writeInitializationDataToPacket( char * networkOrderedBuffer );

	void readDataFromPacket(const char * networkOrderedBuffer );
	void saveDataToPacket( char * networkOrderedBuffer );

	Enemy & operator=(Enemy & _other);

	// well-known constants for each instance of this class:
	// (we don't need to update them through networking)

	// in units per second
	const float mMaxSpeed;
	const float mMaxAcceleration;

protected:

	Ogre::String mId;

	Ogre::Vector3		mVelocity;
	Ogre::Vector3		mAcceleration;

private:

	bool mIsInitialized;
};

#endif
