
#include <Ogre.h>
#include <string.h>
#include <Winsock2.h>

#include "Player.h"
#include "packetID.h"
#include "ParsingUtils.h"

// ============================================================================
Ogre::Vector3 Player::getUpDirection()
{
	return (mSceneNode->getOrientation() * Ogre::Vector3::UNIT_Y);
}
// ============================================================================
Ogre::Vector3 Player::getFacingDirection()
{
	return (mSceneNode->getOrientation() * Ogre::Vector3::UNIT_Z);
}
// ============================================================================
void Player::particleUpdate()
{
	int eRate = 0;

	float partFactor = mAcceleration.length() / mMaxAcceleration;
	eRate = 200 * partFactor;
	eRate += 100;
	if (eRate >200)
	{
		eRate =200;
	}
	if ( mAcceleration.length() / mMaxAcceleration < 0.05) 
	{
		eRate = 0;
	}

	mParticleSys->getEmitter(0)->setEmissionRate(eRate);
}
// ============================================================================
void Player::processAccceleration(unsigned long timeSinceInMilis, float multiplier)
{
	Ogre::Vector3 globalFacingDirection = this->getFacingDirection();
	globalFacingDirection.normalise();

	Ogre::Vector3 globalAcceleration = multiplier * globalFacingDirection * 
								mMaxAcceleration * (float)(timeSinceInMilis)/1000;

	mAcceleration +=  globalAcceleration;

	if(mAcceleration.length() > mMaxAcceleration)
	{
		mAcceleration.normalise();
		mAcceleration *= mMaxAcceleration;
	}
}
// ============================================================================
void Player::processVelocity(unsigned long timeSinceInMilis)
{
	mVelocity += mAcceleration;
	
	if(mVelocity.length() >= mMaxSpeed)
	{
		mVelocity.normalise();
		mVelocity *= mMaxSpeed;
	}
}
// ============================================================================
void Player::processPosition(unsigned long timeSinceInMilis)
{
	mSceneNode->translate(mVelocity);
}
// ============================================================================
void Player::processPitchChange(unsigned long timeSinceInMilis, float multiplier)
{
	mCurrentPitchRotationSpeed += multiplier * mMaxPitchRotationSpeed *
								(float)(timeSinceInMilis)/1000;

	if(mCurrentPitchRotationSpeed < -mMaxPitchRotationSpeed)
	{
		mCurrentPitchRotationSpeed = -mMaxPitchRotationSpeed;
	}

	if(mCurrentPitchRotationSpeed > mMaxPitchRotationSpeed)
	{
		mCurrentPitchRotationSpeed = mMaxPitchRotationSpeed;
	}
}
// ============================================================================
void Player::processRollChange(unsigned long timeSinceInMilis, float multiplier)
{
	mCurrentRollRotationSpeed += multiplier * mMaxRollRotationSpeed *
								(float)(timeSinceInMilis)/1000;

	if(mCurrentRollRotationSpeed < -mMaxRollRotationSpeed)
	{
		mCurrentRollRotationSpeed = -mMaxRollRotationSpeed;
	}

	if(mCurrentRollRotationSpeed > mMaxRollRotationSpeed)
	{
		mCurrentRollRotationSpeed = mMaxRollRotationSpeed;
	}
}
// ============================================================================
void Player::processYawChange(unsigned long timeSinceInMilis, float multiplier)
{
	mCurrentYawRotationSpeed += multiplier * mMaxYawRotationSpeed *
								(float)(timeSinceInMilis)/1000;

	if(mCurrentYawRotationSpeed < -mMaxYawRotationSpeed)
	{
		mCurrentYawRotationSpeed = -mMaxYawRotationSpeed;
	}

	if(mCurrentYawRotationSpeed > mMaxYawRotationSpeed)
	{
		mCurrentYawRotationSpeed = mMaxYawRotationSpeed;
	}
}
// ============================================================================
void Player::processPitch(unsigned long timeSinceInMilis)
{
	mSceneNode->pitch(Ogre::Degree(mCurrentPitchRotationSpeed));
}
// ============================================================================
void Player::processRoll(unsigned long timeSinceInMilis)
{
	mSceneNode->roll(Ogre::Degree(mCurrentRollRotationSpeed));
}
// ============================================================================
void Player::processYaw(unsigned long timeSinceInMilis)
{
	mSceneNode->yaw(Ogre::Degree(mCurrentYawRotationSpeed));
}
// ============================================================================
void Player::processRotationBrake(unsigned long timeSinceInMilis)
{
	float timer = (float)(timeSinceInMilis)/1000;

	// pitch
	float maxPitchStep = timer * mMaxPitchRotationSpeed;
	if(Ogre::Math::Abs(mCurrentPitchRotationSpeed) <= maxPitchStep)
	{
		mCurrentPitchRotationSpeed = 0.0f;
	}
	else if(mCurrentPitchRotationSpeed > 0.0f)
	{
		mCurrentPitchRotationSpeed -= maxPitchStep;
	}
	else
	{
		mCurrentPitchRotationSpeed += maxPitchStep;
	}

	// Roll
	float maxRollStep = timer * mMaxRollRotationSpeed;
	if(Ogre::Math::Abs(mCurrentRollRotationSpeed) <= maxRollStep)
	{
		mCurrentRollRotationSpeed = 0.0f;
	}
	else if(mCurrentRollRotationSpeed > 0.0f)
	{
		mCurrentRollRotationSpeed -= maxRollStep;
	}
	else
	{
		mCurrentRollRotationSpeed += maxRollStep;
	}

	// Yaw
	float maxYawStep = timer * mMaxYawRotationSpeed;
	if(Ogre::Math::Abs(mCurrentYawRotationSpeed) <= maxYawStep)
	{
		mCurrentYawRotationSpeed = 0.0f;
	}
	else if(mCurrentYawRotationSpeed > 0.0f)
	{
		mCurrentYawRotationSpeed -= maxYawStep;
	}
	else
	{
		mCurrentYawRotationSpeed += maxYawStep;
	}
}
// ============================================================================
void Player::readDataFromPacket(const char * networkOrderedBuffer )
{
	int cnt = 0;
	// skip PacketID
	cnt += sizeof(char);

	// We should skip player name, or better - if it isn't our current playerName
	// then skip the command (to avoid possible errors?
	char playerName[MAXPLAYERNAMESIZE];
	memset(playerName,0,MAXPLAYERNAMESIZE);
	memcpy(playerName,&networkOrderedBuffer[cnt],MAXPLAYERNAMESIZE);
	cnt += MAXPLAYERNAMESIZE;

	if(mPlayerName.compare(std::string(playerName)) != 0)
		return;

	// Position
	Ogre::Vector3 position;
	ParsingUtils::parseNetworkedVector3ToHost(position,&networkOrderedBuffer[cnt]);
	cnt += 3*sizeof(float);
	mSceneNode->setPosition(position);

	// Orientation
	Ogre::Quaternion orientation;
	ParsingUtils::parseNetworkedQuaternionToHost(orientation,&networkOrderedBuffer[cnt]);
	cnt += 4*sizeof(float);
	mSceneNode->setOrientation(orientation);

	// Velocity
	ParsingUtils::parseNetworkedVector3ToHost(mVelocity,&networkOrderedBuffer[cnt]);
	cnt += 3*sizeof(float);

	// Acceleration
	ParsingUtils::parseNetworkedVector3ToHost(mAcceleration,&networkOrderedBuffer[cnt]);
	cnt += 3*sizeof(float);

	// Current rotation speeds
	ParsingUtils::parseNetworkedFloatToHost(mCurrentPitchRotationSpeed,&networkOrderedBuffer[cnt]);
	cnt += sizeof(float);

	ParsingUtils::parseNetworkedFloatToHost(mCurrentRollRotationSpeed,&networkOrderedBuffer[cnt]);
	cnt += sizeof(float);

	ParsingUtils::parseNetworkedFloatToHost(mCurrentYawRotationSpeed,&networkOrderedBuffer[cnt]);
	cnt += sizeof(float);
}
// ============================================================================
void Player::saveDataToPacket( char * networkOrderedBuffer )
{
	int cnt = 0;

	networkOrderedBuffer[cnt] = ID_PlayerData;
	cnt += sizeof(char);

	// Player name
	memset(&networkOrderedBuffer[cnt],0,MAXPLAYERNAMESIZE);
	memcpy(&networkOrderedBuffer[cnt], mPlayerName.c_str(), mPlayerName.length());
	cnt += MAXPLAYERNAMESIZE;

	// Position
	ParsingUtils::parseHostVector3ToNetwork(&networkOrderedBuffer[cnt],
		Ogre::Vector3(mSceneNode->getPosition()));
	cnt += 3*sizeof(float);

	// Orientation
	ParsingUtils::parseHostQuaternionToNetwork(&networkOrderedBuffer[cnt],
		Ogre::Quaternion(mSceneNode->getOrientation()));
	cnt += 4*sizeof(float);
	
	// Velocity
	ParsingUtils::parseHostVector3ToNetwork(&networkOrderedBuffer[cnt],mVelocity);
	cnt += 3*sizeof(float);

	// Acceleration
	ParsingUtils::parseHostVector3ToNetwork(&networkOrderedBuffer[cnt],mAcceleration);
	cnt += 3*sizeof(float);

	// Current rotation speeds
	ParsingUtils::parseHostFloatToNetwork(&networkOrderedBuffer[cnt],mCurrentPitchRotationSpeed);
	cnt += sizeof(float);

	ParsingUtils::parseHostFloatToNetwork(&networkOrderedBuffer[cnt],mCurrentRollRotationSpeed);
	cnt += sizeof(float);

	ParsingUtils::parseHostFloatToNetwork(&networkOrderedBuffer[cnt],mCurrentYawRotationSpeed);
	cnt += sizeof(float);
}
// ============================================================================
bool Player::writeInitializationDataToPacket( char * networkOrderedBuffer )
{
	int cnt = 0;

	networkOrderedBuffer[cnt] = ID_SendNewPlayerData;
	cnt += sizeof(char);

	// Entity ID
	memset(&networkOrderedBuffer[cnt],0,MAXENTITYIDSIZE);
	memcpy(&networkOrderedBuffer[cnt], mId.c_str(), mId.length());
	cnt += MAXENTITYIDSIZE;

	// Player name
	memset(&networkOrderedBuffer[cnt],0,MAXPLAYERNAMESIZE);
	memcpy(&networkOrderedBuffer[cnt], mPlayerName.c_str(), mPlayerName.length());
	cnt += MAXPLAYERNAMESIZE;

	// Colour
	networkOrderedBuffer[cnt] = mColour;
	cnt += sizeof(char);

	// Position
	ParsingUtils::parseHostVector3ToNetwork(&networkOrderedBuffer[cnt],
		Ogre::Vector3(mSceneNode->getPosition()));
	cnt += 3*sizeof(float);

	// Orientation
	ParsingUtils::parseHostQuaternionToNetwork(&networkOrderedBuffer[cnt],
		Ogre::Quaternion(mSceneNode->getOrientation()));
	cnt += 4*sizeof(float);
	
	// Velocity
	ParsingUtils::parseHostVector3ToNetwork(&networkOrderedBuffer[cnt],mVelocity);
	cnt += 3*sizeof(float);

	// Acceleration
	ParsingUtils::parseHostVector3ToNetwork(&networkOrderedBuffer[cnt],mAcceleration);
	cnt += 3*sizeof(float);

	// Current rotation speeds
	ParsingUtils::parseHostFloatToNetwork(&networkOrderedBuffer[cnt],mCurrentPitchRotationSpeed);
	cnt += sizeof(float);

	ParsingUtils::parseHostFloatToNetwork(&networkOrderedBuffer[cnt],mCurrentRollRotationSpeed);
	cnt += sizeof(float);

	ParsingUtils::parseHostFloatToNetwork(&networkOrderedBuffer[cnt],mCurrentYawRotationSpeed);
	cnt += sizeof(float);

	// yeah, its like SaveDataToPacket but could do more, like send some initial score, lives etc

	// ... which we are doing now:
	// Score:
	ParsingUtils::packi32((unsigned char *)&networkOrderedBuffer[cnt], mScore);
	cnt += sizeof(int);

	// Lives:
	ParsingUtils::packi32((unsigned char *)&networkOrderedBuffer[cnt], mLives);
	cnt += sizeof(int);

	
	return true;
}
// ============================================================================
bool Player::initializeFromPacket( const char * networkOrderedBuffer, Ogre::String sceneManager, bool smallerParticles )
{
	int cnt = 0;
	// skip PacketID
	cnt += sizeof(char);

	// Entity ID + mesh and SceneNode init
	char entityId[MAXENTITYIDSIZE];
	memset(entityId,0,MAXENTITYIDSIZE);
	memcpy(entityId,&networkOrderedBuffer[cnt],MAXENTITYIDSIZE);
	mId = std::string(entityId);
	cnt += MAXENTITYIDSIZE;

	if(!mMesh)
	{
		mMesh = Ogre::Root::getSingleton().getSceneManager(
			sceneManager)->createEntity(mId, "Oort.mesh");
	}

	if(!mSceneNode)
	{
		mSceneNode = Ogre::Root::getSingleton().getSceneManager(
			sceneManager)->getRootSceneNode()->createChildSceneNode(mId+"Node");

		mSceneNode->attachObject(mMesh);
		mSceneNode->setScale( Ogre::Vector3(1.0,1.0,1.0) );
	}

	if(mSceneNode)
	{
		mParticleNode = mSceneNode->createChildSceneNode();
		if(smallerParticles == true)
		{
			mParticleSys = Ogre::Root::getSingleton().getSceneManager(
				"PlayState SceneManager")->createParticleSystem(mId+"Particle", "tail2-local");
		}
		else
		{
			mParticleSys = Ogre::Root::getSingleton().getSceneManager(
				"PlayState SceneManager")->createParticleSystem(mId+"Particle", "tail2");
		}
		mParticleNode->setPosition(Ogre::Vector3(0, -10, 70));
		mParticleNode->attachObject(mParticleSys);
	}

	// Player name
	char playerName[MAXPLAYERNAMESIZE];
	memset(playerName,0,MAXPLAYERNAMESIZE);
	memcpy(playerName,&networkOrderedBuffer[cnt],MAXPLAYERNAMESIZE);
	mPlayerName = std::string(playerName);
	cnt += MAXPLAYERNAMESIZE;

	// Colour + model material
	char mColour = networkOrderedBuffer[cnt];
	cnt += sizeof(char);

	this->setColour(mColour);

	// Position
	Ogre::Vector3 position;
	ParsingUtils::parseNetworkedVector3ToHost(position,&networkOrderedBuffer[cnt]);
	cnt += 3*sizeof(float);
	mSceneNode->setPosition(position);

	// Orientation
	Ogre::Quaternion orientation;
	ParsingUtils::parseNetworkedQuaternionToHost(orientation,&networkOrderedBuffer[cnt]);
	cnt += 4*sizeof(float);
	mSceneNode->setOrientation(orientation);

	// Velocity
	ParsingUtils::parseNetworkedVector3ToHost(mVelocity,&networkOrderedBuffer[cnt]);
	cnt += 3*sizeof(float);

	// Acceleration
	ParsingUtils::parseNetworkedVector3ToHost(mAcceleration,&networkOrderedBuffer[cnt]);
	cnt += 3*sizeof(float);

	// Current rotation speeds
	ParsingUtils::parseNetworkedFloatToHost(mCurrentPitchRotationSpeed,&networkOrderedBuffer[cnt]);
	cnt += sizeof(float);

	ParsingUtils::parseNetworkedFloatToHost(mCurrentRollRotationSpeed,&networkOrderedBuffer[cnt]);
	cnt += sizeof(float);

	ParsingUtils::parseNetworkedFloatToHost(mCurrentYawRotationSpeed,&networkOrderedBuffer[cnt]);
	cnt += sizeof(float);

	// Score
	mScore = ParsingUtils::unpacki32((unsigned char *)&networkOrderedBuffer[cnt]);
	cnt += sizeof(int);

	// Lives
	mLives = ParsingUtils::unpacki32((unsigned char *)&networkOrderedBuffer[cnt]);
	cnt += sizeof(int);


	mIsInitialized = true;
	
	// No problems at all 
	return true;
}
// ============================================================================
void Player::setColour(char colour)
{
	if(!mMesh)
		return;

	switch(colour)
	{
		case 'G':
			mColour = 'G';
			mMesh->setMaterialName("OortGreen");
			break;
		case 'R':
			mColour = 'R';
			mMesh->setMaterialName("OortRed");
			break;
		case 'W':
			mColour = 'W';
			mMesh->setMaterialName("OortWhite");
			break;
	}

}
// ============================================================================
void Player::setPosition(Ogre::Vector3 & _position)
{
	mSceneNode->setPosition(_position);
}
// ============================================================================
void Player::setOrientation(Ogre::Quaternion & _orientation)
{
	mSceneNode->setOrientation(_orientation);
}
// ============================================================================
Player::Player(void) :
	mClassName("Player"),
	mMesh(0),
	mSceneNode(0),
	mMaxSpeed(10),
	mMaxAcceleration(0.5),
	mMaxRotationAcceleration(1),
	mMaxPitchRotationSpeed(5),
	mMaxRollRotationSpeed(5),
	mMaxYawRotationSpeed(5),
	mId(""),
	mPlayerName("UninitializedPlayer"),
	mIsInitialized(false),
	mParticleNode(0),
	mParticleSys(0),
	mFiringDelayInMilis(300),
	mMilisSinceLastFiring(mFiringDelayInMilis),
	mScore(0),
	mLives(0)
{
}
// ============================================================================
void Player::setId(Ogre::String _id)
{
	mId = _id;
}
// ============================================================================
void Player::setPlayerName(Ogre::String _playerName)
{
	mPlayerName = _playerName;
}
// ============================================================================
void Player::setVelocity(Ogre::Vector3 & _velocity)
{
	mVelocity = _velocity;
}
// ============================================================================
void Player::setAcceleration(Ogre::Vector3 & _acceleration)
{
	mAcceleration = _acceleration;
}
// ============================================================================
void Player::setCurrentPitchRotationSpeed(float _pitchRotSpeed)
{
	mCurrentPitchRotationSpeed = _pitchRotSpeed;
}
// ============================================================================
void Player::setCurrentRollRotationSpeed(float _rollRotSpeed)

{
	mCurrentRollRotationSpeed = _rollRotSpeed;
}
// ============================================================================
void Player::setCurrentYawRotationSpeed(float _yawRotSpeed)
{
	mCurrentYawRotationSpeed = _yawRotSpeed;
}
// ============================================================================
Player & Player::operator=(Player _other)
{
	mMesh = _other.mMesh;
	mSceneNode = _other.mSceneNode;

	mParticleNode = _other.mParticleNode;
	mParticleSys = _other.mParticleSys;

	mId = _other.mId;
	mPlayerName = _other.mPlayerName;

	mColour = _other.mColour;

	mVelocity = _other.mVelocity;
	mAcceleration = _other.mAcceleration;

	mCurrentPitchRotationSpeed = _other.mCurrentPitchRotationSpeed;
	mCurrentRollRotationSpeed = _other.mCurrentRollRotationSpeed;
	mCurrentYawRotationSpeed = _other.mCurrentYawRotationSpeed;

	mScore = _other.mScore;
	mLives = _other.mLives;

	mIsInitialized = _other.mIsInitialized;

	return *this;
}
// ============================================================================
void Player::setScore(int _newScore)
{
	mScore = _newScore;
}
// ============================================================================
void Player::setLives(int _newLives)
{
	mLives = _newLives;
}
// ============================================================================
void Player::readScoreFromPacket(const char * networkOrderedBuffer )
{
	int cnt = 0;
	// skip PacketID
	cnt += sizeof(char);

	std::string playerName(&networkOrderedBuffer[cnt]);
	cnt += MAXPLAYERNAMESIZE;

	if(mPlayerName.compare(playerName) != 0)
		return;

	// Score
	mScore = ParsingUtils::unpacki32((unsigned char *)&networkOrderedBuffer[cnt]);
	cnt += sizeof(int);
}
// ============================================================================
void Player::saveScoreToPacket( char * networkOrderedBuffer )
{
	int cnt = 0;

	networkOrderedBuffer[cnt] = ID_ServerChangesPlayerScore;
	cnt += sizeof(char);

	// Player name
	memset(&networkOrderedBuffer[cnt],0,MAXPLAYERNAMESIZE);
	memcpy(&networkOrderedBuffer[cnt], mPlayerName.c_str(), mPlayerName.length());
	cnt += MAXPLAYERNAMESIZE;

	// Score:
	ParsingUtils::packi32((unsigned char *)&networkOrderedBuffer[cnt], mScore);
	cnt += sizeof(int);
}
// ============================================================================
void Player::readLivesFromPacket(const char * networkOrderedBuffer )
{
	int cnt = 0;
	// skip PacketID
	cnt += sizeof(char);

	std::string playerName(&networkOrderedBuffer[cnt]);
	cnt += MAXPLAYERNAMESIZE;

	if(mPlayerName.compare(playerName) != 0)
		return;

	// Score
	mLives = ParsingUtils::unpacki32((unsigned char *)&networkOrderedBuffer[cnt]);
	cnt += sizeof(int);
}
// ============================================================================
void Player::saveLivesToPacket( char * networkOrderedBuffer )
{
	int cnt = 0;

	networkOrderedBuffer[cnt] = ID_ServerChangesPlayerLives;
	cnt += sizeof(char);

	// Player name
	memset(&networkOrderedBuffer[cnt],0,MAXPLAYERNAMESIZE);
	memcpy(&networkOrderedBuffer[cnt], mPlayerName.c_str(), mPlayerName.length());
	cnt += MAXPLAYERNAMESIZE;

	// Score:
	ParsingUtils::packi32((unsigned char *)&networkOrderedBuffer[cnt], mLives);
	cnt += sizeof(int);
}
// ============================================================================