
#ifndef Player_H
#define Player_H

#include <Ogre.h>

class Player
{
public:
	Player(void);
	~Player(void) {};


	const Ogre::String		mClassName;

	Ogre::Entity		*mMesh;
	Ogre::SceneNode		*mSceneNode;

	Ogre::String	getId() { return mId; };
	void			setId(Ogre::String _id);

	Ogre::String	getPlayerName() { return mPlayerName; };
	void			setPlayerName(Ogre::String _playerName);

	char			getColour() { return mColour; };
	void			setColour(char); //special

	Ogre::Vector3	getPosition() { return mSceneNode->getPosition(); };
	void			setPosition(Ogre::Vector3 & _position);

	Ogre::Quaternion getOrientation() { return mSceneNode->getOrientation(); };
	void			setOrientation(Ogre::Quaternion & _orientation);

	Ogre::Vector3	getVelocity() { return mVelocity; };
	const float getZVelocity () { return (int)((mVelocity.x*mVelocity.x) + (mVelocity.y*mVelocity.y) + (mVelocity.z*mVelocity.z)); };
	void			setVelocity(Ogre::Vector3 & _velocity);

	Ogre::Vector3	getAcceleration() { return mAcceleration; }
	void			setAcceleration(Ogre::Vector3 & _acceleration);

	float			getCurrentPitchRotationSpeed() { return mCurrentPitchRotationSpeed; };
	void			setCurrentPitchRotationSpeed(float _pitchRotSpeed);

	float			getCurrentRollRotationSpeed() { return mCurrentRollRotationSpeed; };
	void			setCurrentRollRotationSpeed(float _rollRotSpeed);

	float			getCurrentYawRotationSpeed() { return mCurrentYawRotationSpeed; };
	void			setCurrentYawRotationSpeed(float _yawRotSpeed);

	int				getScore() { return mScore; };
	void			setScore(int _newScore);

	int				getLives() { return mLives; };
	void			setLives(int _newLives);

	Ogre::Vector3	getFacingDirection();
	Ogre::Vector3	getUpDirection();

	// Movement functions (ease of use)

	// Accelerate in our local direction
	void processAccceleration(unsigned long timeSinceInMilis, float multiplier = 1.0f);

	// Change the velocity according to the new acceleration
	void processVelocity(unsigned long timeSinceInMilis);
	
	// Apply the velocity to change position
	void processPosition(unsigned long timeSinceInMilis);

	// Process rotations
	void processPitchChange(unsigned long timeSinceInMilis, float multiplier = 1.0f);
	void processRollChange(unsigned long timeSinceInMilis, float multiplier = 1.0f);
	void processYawChange(unsigned long timeSinceInMilis, float multiplier = 1.0f);

	void processPitch(unsigned long timeSinceInMilis);
	void processRoll(unsigned long timeSinceInMilis);
	void processYaw(unsigned long timeSinceInMilis);

	// Process rotation halt
	void processRotationBrake(unsigned long timeSinceInMilis);

	// Creating a player from a network packet
	bool initializeFromPacket( const char * networkOrderedBuffer, Ogre::String sceneManager, bool smallerParticles = false );

	bool isInitialized() { return mIsInitialized; };

	// Send the creation (position, Player name, ID, etc) through the network
	bool writeInitializationDataToPacket( char * networkOrderedBuffer );

	void readDataFromPacket(const char * networkOrderedBuffer );
	void saveDataToPacket( char * networkOrderedBuffer );

	void readScoreFromPacket(const char * networkOrderedBuffer );
	void saveScoreToPacket( char * networkOrderedBuffer );

	void readLivesFromPacket(const char * networkOrderedBuffer );
	void saveLivesToPacket( char * networkOrderedBuffer );

	Player & operator=(Player _other);

	// well-known constants for each instance of this class:
	// (we don't need to update them through networking)

	// in units per second
	const float mMaxSpeed;
	const float mMaxAcceleration;

	// in Degrees per second
	const float mMaxRotationAcceleration;
	const float mMaxPitchRotationSpeed;
	const float mMaxRollRotationSpeed;
	const float mMaxYawRotationSpeed;


	Ogre::SceneNode * mParticleNode;
	Ogre::ParticleSystem * mParticleSys;

	// Updates the particles
	void particleUpdate();

	const float mFiringDelayInMilis;
	float mMilisSinceLastFiring;

	bool mIsInitialized;

protected:

	// Server-updateable: (ie serializable)

	// Our uniqe Model Entity ID, like "OortGreen"
	// It is used to send and receive network data
	// (usually constant)
	Ogre::String mId;

	// Player name, like "Pwn0r Playa 3000"
	// (can change)
	Ogre::String mPlayerName;

	char mColour;

	//Ogre::Vector3		mPosition; // <- included in SceneNode
	//Ogre::Quaternion	mOrientation; // <- included in SceneNode
	Ogre::Vector3		mVelocity;
	Ogre::Vector3		mAcceleration;

	// Degrees per second
	float mCurrentPitchRotationSpeed;
	float mCurrentRollRotationSpeed;
	float mCurrentYawRotationSpeed;

	// Player score
	int mScore;

	// Player lives
	int mLives;

};

#endif
